# -*- coding: utf-8 -*-
from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from collections import OrderedDict as BaseDict
from collective.easyform.config import MODEL_DEFAULT
from email.utils import formataddr
from hashlib import md5
from plone.memoize import ram
from plone.namedfile.interfaces import INamedBlobFile
from plone.namedfile.interfaces import INamedFile
from plone.supermodel import loadString
from plone.supermodel import serializeSchema
from Products.CMFCore.Expression import Expression
from Products.CMFCore.Expression import getExprContext
from Products.CMFPlone.utils import safe_unicode
from re import compile
from types import StringTypes
from zope.schema import getFieldsInOrder


# SCHEMATA_KEY = u''
CONTEXT_KEY = u'context'
# regular expression for dollar-sign variable replacement.
# we want to find ${identifier} patterns
dollarRE = compile(r'\$\{(.+?)\}')


class OrderedDict(BaseDict):
    """
    A wrapper around dictionary objects that provides an ordering for
    keys() and items().
    """

    security = ClassSecurityInfo()
    security.setDefaultAccess("allow")

    def reverse(self):
        items = list(self.items())
        items.reverse()
        return items


InitializeClass(OrderedDict)


class DollarVarReplacer(object):

    """
    Initialize with a dictionary, then self.sub returns a string
    with all ${key} substrings replaced with values looked
    up from the dictionary.

    >>> from collective.easyform import api

    >>> adict = {'one':'two', '_two':'three', '.two':'four'}
    >>> dvr = api.DollarVarReplacer(adict)

    >>> dvr.sub('one one')
    'one one'

    >>> dvr.sub('one ${one}')
    'one two'

    >>> dvr.sub('one ${two}')
    'one ???'

    Skip any key beginning with _ or .
    >>> dvr.sub('one ${_two}')
    'one ???'

    >>> dvr.sub('one ${.two}')
    'one ???'

    """

    def __init__(self, adict):
        self.adict = adict

    def sub(self, s):
        return dollarRE.sub(self.repl, s)

    def repl(self, mo):
        key = mo.group(1)
        if key and key[0] not in ['_', '.']:
            try:
                return self.adict[mo.group(1)]
            except KeyError:
                pass
        return '???'


def get_expression(context, expression_string, **kwargs):
    """ Get TALES expression

    :param context: [required] TALES expression context
    :param string expression_string: [required] TALES expression string
    :param dict kwargs: additional arguments for expression
    :returns: result of TALES expression
    """
    if isinstance(expression_string, unicode):
        expression_string = expression_string.encode('utf-8')

    expression_context = getExprContext(context, context)
    for key in kwargs:
        expression_context.setGlobal(key, kwargs[key])
    expression = Expression(expression_string)
    return expression(expression_context)


def get_context(field):
    """ Get context of field

    :param field: [required] easyform field
    :returns: easyform
    """
    return field.interface.getTaggedValue(CONTEXT_KEY)


def get_fields_cache(method, context):
    data = context.fields_model + str(context.modification_date)
    if isinstance(data, unicode):
        data = data.encode('utf-8')
    return md5(data).hexdigest()


@ram.cache(get_fields_cache)
def get_fields(context):
    data = context.fields_model
    try:
        schema = loadString(data).schema
    except Exception:
        schema = loadString(MODEL_DEFAULT).schema
    schema.setTaggedValue(CONTEXT_KEY, context)
    return schema


def get_actions_cache(method, context):
    data = context.actions_model + str(context.modification_date)
    return md5(data).hexdigest()


@ram.cache(get_actions_cache)
def get_actions(context):
    data = context.actions_model
    try:
        schema = loadString(data).schema
    except Exception:
        schema = loadString(MODEL_DEFAULT).schema
    schema.setTaggedValue(CONTEXT_KEY, context)
    return schema


def set_fields(context, schema):
    # serialize the current schema
    snew_schema = serializeSchema(schema)
    # store the current schema
    context.fields_model = snew_schema
    context.notifyModified()


def set_actions(context, schema):
    # fix setting widgets
    schema.setTaggedValue('plone.autoform.widgets', {})
    # serialize the current schema
    snew_schema = serializeSchema(schema)
    # store the current schema
    context.actions_model = snew_schema
    context.notifyModified()


def format_addresses(addresses, names=[]):
    """Format destination (To) input.
    Input may be a string or sequence of strings;
    returns a well-formatted address string.

    :param addresses: [required] Mail addresses.
    :type addresses: sequence or string
    :param names: Names corresponding to mail addresses.
    :type addresses: sequence or string
    :returns: Well formatted address string.
    :rtype: String

    >>> from collective.easyform import api

    >>> api.format_addresses('sim@sala.bim')
    'sim@sala.bim'

    >>> api.format_addresses('sim@sala.bim', 'sim')
    'sim <sim@sala.bim>'

    >>> api.format_addresses('ähm@öhm.ühm', 'ähm')
    '\xc3\xa4hm <\xc3\xa4hm@\xc3\xb6hm.\xc3\xbchm>'

    >>> api.format_addresses('sim@sala.bim, hokus@pokus.fidibus')
    'sim@sala.bim, hokus@pokus.fidibus'

    >>> api.format_addresses('sim@sala.bim, hokus@pokus.fidibus', 'sim')
    'sim <sim@sala.bim>, hokus@pokus.fidibus'

    >>> api.format_addresses('sim@sala.bim, hokus@pokus.fidibus', 'sim, hokus')
    'sim <sim@sala.bim>, hokus <hokus@pokus.fidibus>'

    >>> api.format_addresses(['sim@sala.bim', 'hokus@pokus.fidibus'],
    ...                      ['sim', 'hokus'])
    'sim <sim@sala.bim>, hokus <hokus@pokus.fidibus>'

    >>> api.format_addresses(('sim@sala.bim', 'hokus@pokus.fidibus'),
    ...                      ('sim', 'hokus'))
    'sim <sim@sala.bim>, hokus <hokus@pokus.fidibus>'

    >>> api.format_addresses([])
    ''

    >>> api.format_addresses('')
    ''

    """

    if type(addresses) in StringTypes:
        addresses = [s for s in addresses.split(',')]
    assert(isinstance(addresses, list) or isinstance(addresses, tuple))  # ensure iterable  # noqa
    addresses = [
        safe_unicode(s).strip().encode('utf-8') for s in addresses if s
    ]

    if names and type(names) in StringTypes:
        names = [s for s in names.split(',')]
    if not names:
        names = []
    assert(isinstance(names, list) or isinstance(names, tuple))
    names = [safe_unicode(s).strip().encode('utf-8') for s in names if s]

    address_pairs = []
    for cnt, address in enumerate(addresses):
        address_pairs.append((
            names[cnt] if len(names) > cnt else False,
            address
        ))

    ret = ', '.join([formataddr(pair) for pair in address_pairs])
    return ret


def is_file_data(field):
    """Return True, if field is a file field.
    """
    ifaces = (INamedFile, INamedBlobFile)
    for i in ifaces:
        if i.providedBy(field):
            return True
    return False


def filter_fields(context, schema, unsorted_data, omit=False):
    """Filter according to ``showAll``, ``showFields`` and ``includeEmpties``
    settings to display in result mailings, thanks pages and the like.
    """
    # Prevent cyclic import
    from collective.easyform.interfaces import IFieldExtender

    data = OrderedDict(
        [
            (x[0], unsorted_data[x[0]])
            for x in getFieldsInOrder(schema)
            if x[0] in unsorted_data
        ]
    )

    # TODO: Exclude labels
    fields = [
        f
        for f in data
        if not (is_file_data(data[f]))
        and not (
            # TODO: serverSide should always be excluded
            # Also, when shoAll = 0 and serverSite = 1, it will be included
            # which is wrong.
            getattr(context, "showAll", True)
            and IFieldExtender(schema[f]).serverSide
        )
    ]

    if not getattr(context, "showAll", True):
        showFields = getattr(context, "showFields", []) or []
        # Only show fields from 'showFields', and also keep the order,
        # even when that is not always needed or used by code that calls us.
        # Note that showFields may contain fields that are not in the schema
        # or the data: the schema may be limited to one fieldset.
        # fields = [f for f in fields if f in showFields]
        ordered_fields = []
        for fieldname in showFields:
            for f in fields:
                if f == fieldname:
                    ordered_fields.append(f)
                    fields.remove(f)
                    break
        fields = ordered_fields

    if not getattr(context, "includeEmpties", True):
        fields = [f for f in fields if data[f]]

    ret = []
    if omit:
        # return field ids to omit
        ret = [f for f in data if f not in fields]
    else:
        # return field ids and field instances to include
        ret = OrderedDict([(f, data[f]) for f in fields])

    return ret



# -*- coding: utf-8 -*-
from Testing import ZopeTestCase as ztc
from collective.easyform.tests.base import EasyFormFunctionalTestCase
# from plone.testing import layered
import doctest
import unittest2 as unittest

optionflags = (
    doctest.REPORT_ONLY_FIRST_FAILURE |
    doctest.NORMALIZE_WHITESPACE |
    doctest.ELLIPSIS |
    doctest.REPORTING_FLAGS)

testfiles = (
    'browser.txt',
    'attachment.txt',
    'ssl.txt',
    'serverside_field.txt',
    '../api.py',
    '../README.txt',
)

# easyformfunctionaltestcase = EasyFormFunctionalTestCase()
# def setUp(test):
#     easyformfunctionaltestcase.setUp()
# def tearDown(test):
#     easyformfunctionaltestcase.tearDown()

# def test_suite():
#     suite = unittest.TestSuite()
#     suite.addTests([
#         layered(doctest.DocFileSuite(test,
#                                      optionflags=optionflags,
#                                      setUp=setUp,
#                                      tearDown=tearDown,
#                                      ),
#                 layer=EasyFormFunctionalTestCase.layer)
#         for test in testfiles])
#     return suite


def test_suite():
    return unittest.TestSuite([
        ztc.FunctionalDocFileSuite(
            f, package='collective.easyform.tests',
            test_class=EasyFormFunctionalTestCase,
            optionflags=optionflags)
        for f in testfiles
    ])



# -*- coding: utf-8 -*-
#
# Test EasyForm initialisation and set-up
#

from AccessControl import Unauthorized
from Products.CMFCore.utils import getToolByName
from collective.easyform.tests import base
import Products


def getAddPermission(product, name):
    ''' find the add permission for a meta_type '''

    name = '{0}: {1}'.format(product, name)
    for mt in Products.meta_types:
        if mt['name'] == name:
            return mt['permission']
    return ''


class TestInstallation(base.EasyFormTestCase):

    '''Ensure product is properly installed'''

    def afterSetUp(self):
        base.EasyFormTestCase.afterSetUp(self)

        self.types = self.portal.portal_types
        self.properties = self.portal.portal_properties
        self.at_tool = self.portal.archetype_tool
        self.controlpanel = self.portal.portal_controlpanel

        self.metaTypes = ('EasyForm',)

    def testTypesInstalled(self):
        for t in self.metaTypes:
            self.assertTrue(t in self.types.objectIds())

    def testTypeActions(self):
        # hide properties/references tabs
        for typ in self.metaTypes:
            for act in self.types[typ].listActions():
                if act.id in ['metadata', 'references']:
                    self.assertFalse(act.visible)
                if act.id in ['actions', 'fields']:
                    self.assertTrue(act.visible)

    def testArchetypesToolCatalogRegistration(self):
        for t in self.metaTypes:
            self.assertEqual(1, len(self.at_tool.getCatalogsByType(t)))
            self.assertEqual(
                'portal_catalog', self.at_tool.getCatalogsByType(t)[0].getId())

    def ttestControlPanelConfigletInstalled(self):
        self.assertTrue(
            'EasyForm' in [action.id for action in self.controlpanel.listActions()])

    def ttestAddPermissions(self):
        ''' Test to make sure add permissions are as intended '''

        ADD_CONTENT_PERMISSION = 'EasyForm: Add Content'
        CSA_ADD_CONTENT_PERMISSION = 'EasyForm: Add Custom Scripts'
        MA_ADD_CONTENT_PERMISSION = 'EasyForm: Add Mailers'
        SDA_ADD_CONTENT_PERMISSION = 'EasyForm: Add Data Savers'

        self.assertEqual(
            getAddPermission('EasyForm', 'EasyForm'), ADD_CONTENT_PERMISSION)
        self.assertEqual(
            getAddPermission('EasyForm', 'Mailer Adapter'), MA_ADD_CONTENT_PERMISSION)
        self.assertEqual(
            getAddPermission('EasyForm', 'Save Data Adapter'), SDA_ADD_CONTENT_PERMISSION)
        self.assertEqual(
            getAddPermission('EasyForm', 'Custom Script Adapter'), CSA_ADD_CONTENT_PERMISSION)

    def testActionsInstalled(self):
        # self.setRoles(['Manager', ])
        self.assertTrue(
            self.portal.portal_actions.getActionInfo('object_buttons/export'))
        self.assertTrue(
            self.portal.portal_actions.getActionInfo('object_buttons/import'))
        self.assertTrue(
            self.portal.portal_actions.getActionInfo('object_buttons/saveddata'))

    def ttest_FormGenTool(self):
        self.assertTrue(getToolByName(self.portal, 'formgen_tool'))

    def testModificationsToPropSheetLinesNotPuged(self):
        property_mappings = [{
            'propsheet': 'site_properties',
            'added_props': [
                'use_folder_tabs',
                'typesLinkToFolderContentsInFC',
                'default_page_types',
            ]
        }]

        # add garbage prop element to each lines property
        for mapping in property_mappings:
            sheet = self.properties[mapping['propsheet']]
            for lines_prop in mapping['added_props']:
                propitems = list(sheet.getProperty(lines_prop))
                propitems.append('foo')
                sheet.manage_changeProperties({lines_prop: propitems})

        # reinstall
        qi = self.portal.portal_quickinstaller
        qi.reinstallProducts(['collective.easyform'])

        # now make sure our garbage values survived the reinstall
        for mapping in property_mappings:
            sheet = self.properties[mapping['propsheet']]
            for lines_prop in mapping['added_props']:
                self.assertTrue('foo' in sheet.getProperty(lines_prop),
                                "Our garbage item didn't survive reinstall for property {0}"
                                " within property sheet {1}".format(lines_prop, mapping['propsheet']))

    def test_EasyFormInDefaultPageTypes(self):
        propsTool = getToolByName(self.portal, 'portal_properties')
        siteProperties = getattr(propsTool, 'site_properties')
        defaultPageTypes = list(
            siteProperties.getProperty('default_page_types'))
        self.assertTrue('EasyForm' in defaultPageTypes)

    def testTypeViews(self):
            self.assertEqual(
                self.types.EasyForm.getAvailableViewMethods(self.types), ('view',))


class TestContentCreation(base.EasyFormTestCase):

    '''Ensure content types can be created and edited'''

    fieldTypes = [
        'FormSelectionField',
        'FormMultiSelectionField',
        'FormLabelField',
        'FormDateField',
        'FormLinesField',
        'FormIntegerField',
        'FormBooleanField',
        'FormPasswordField',
        'FormFixedPointField',
        'FormStringField',
        'FormTextField',
        'FormRichTextField',
        'FormFileField',
    ]
    # if base.haveRecaptcha:
    #    fieldTypes.append('FormCaptchaField')
    fieldTypes = tuple(fieldTypes)

    adapterTypes = (
        'FormSaveDataAdapter',
        'FormMailerAdapter',
    )

    thanksTypes = (
        'FormThanksPage',
    )

    fieldsetTypes = (
        'FieldsetFolder',
    )

    sampleContentIds = ('mailer', 'replyto', 'topic', 'comments', 'thank-you')

    def afterSetUp(self):
        base.EasyFormTestCase.afterSetUp(self)
        self.folder.invokeFactory('EasyForm', 'ff1')
        self.ff1 = getattr(self.folder, 'ff1')

    def testCreateEasyForm(self):
        self.assertTrue('ff1' in self.folder.objectIds())

    def testSampleContent(self):
        # check embedded content
        oi = self.ff1.objectIds()
        for id in self.sampleContentIds:
            self.assertTrue(id in oi)

    def testSampleMailerSetup(self):
        self.assertEqual(self.ff1.actionAdapter, ('mailer',))
        self.assertEqual(self.ff1.mailer.replyto_field, 'replyto')
        self.assertEqual(self.ff1.mailer.subject_field, 'topic')
        self.assertEqual(self.ff1.mailer.thanksPage, 'thank-you')

    def testEasyFormCanSetDefaultPage(self):
        self.assertEqual(self.ff1.canSetDefaultPage(), False)

    def testEditEasyForm(self):
        self.ff1.setTitle('A title')
        self.ff1.setDescription('A description')

        self.assertEqual(self.ff1.Title(), 'A title')
        self.assertEqual(self.ff1.Description(), 'A description')

    def testCreateFields(self):
        for f in self.fieldTypes:
            fname = '{0}1'.format(f)
            self.ff1.invokeFactory(f, fname)
            self.assertTrue(fname in self.ff1.objectIds())

    def testCreateAdapters(self):
        for f in self.adapterTypes:
            fname = '{0}1'.format(f)
            self.ff1.invokeFactory(f, fname)
            self.assertTrue(fname in self.ff1.objectIds())
            self.assertTrue(hasattr(self.ff1[fname], 'onSuccess'))

    def testCreateThanksPages(self):
        for f in self.thanksTypes:
            fname = '{0}1'.format(f)
            self.ff1.invokeFactory(f, fname)
            self.assertTrue(fname in self.ff1.objectIds())
            self.assertTrue(hasattr(self.ff1[fname], 'displayFields'))

    def testCreateFieldset(self):
        for f in self.fieldsetTypes:
            fname = '{0}1'.format(f)
            self.ff1.invokeFactory(f, fname)
            self.assertTrue(fname in self.ff1.objectIds())

    def testCreateFieldsinFieldset(self):
        fname = 'FieldsetFolder1'
        self.ff1.invokeFactory('FieldsetFolder', fname)
        self.assertTrue(fname in self.ff1.objectIds())
        fs = self.ff1[fname]
        for f in self.fieldTypes:
            fname = '{0}1fs'.format(f)
            fs.invokeFactory(f, fname)
            self.assertTrue(fname in fs.objectIds())

        # Things that shouldn't fit in a fieldset folder
        self.assertRaises(ValueError, fs.invokeFactory, 'EasyForm', 'ffinf')
        self.assertRaises(
            ValueError, fs.invokeFactory, 'FormThanksPage', 'ffinf')
        self.assertRaises(
            ValueError, fs.invokeFactory, 'FieldsetFolder', 'ffinf')
        self.assertRaises(
            ValueError, fs.invokeFactory, 'FormMailerAdapter', 'ffinf')

        # try finding the fields
        for f in self.fieldTypes:
            fname = '{0}1fs'.format(f)
            self.assertTrue(self.ff1.findFieldObjectByName(fname))

    def testFgFieldsDisplayOnly(self):
        ''' Make sure fgFields displayOnly parameter correctly excludes
            labels and fieldset markers
        '''

        noExtras = len(self.ff1.fgFields(displayOnly=True))
        wExtras = len(self.ff1.fgFields())
        self.assertTrueEqual(wExtras, noExtras)

        self.ff1.invokeFactory('FieldsetFolder', 'FieldsetFolder1')
        self.ff1['FieldsetFolder1'].invokeFactory('FormStringField', 'fsf')

        noExtras = len(self.ff1.fgFields(displayOnly=True))
        wExtras = len(self.ff1.fgFields())
        self.assertTrueEqual(wExtras, noExtras + 2)

        self.ff1.invokeFactory('FormLabelField', 'flf')
        noExtras = len(self.ff1.fgFields(displayOnly=True))
        wExtras = len(self.ff1.fgFields())
        self.assertTrueEqual(wExtras, noExtras + 3)

        self.ff1.invokeFactory('FormRichLabelField', 'frlf')
        noExtras = len(self.ff1.fgFields(displayOnly=True))
        wExtras = len(self.ff1.fgFields())
        self.assertTrueEqual(wExtras, noExtras + 4)

    def testEditField(self):
        for f in self.fieldTypes:
            fname = '{0}1'.format(f)
            self.ff1.invokeFactory(f, fname)
            f1 = getattr(self.ff1, fname)
            f1.setTitle('Field title')
            f1.setDescription('Field description')

            self.assertEqual(f1.Title(), 'Field title')
            self.assertEqual(f1.fgField.widget.label, 'Field title')
            self.assertEqual(f1.Description(), 'Field description')
            self.assertEqual(
                f1.fgField.widget.description, 'Field description')

    def testTALESFieldValidation(self):
        for f in self.fieldTypes:
            if f != 'FormLabelField':
                fname = '{0}1'.format(f)
                self.ff1.invokeFactory(f, fname)
                f1 = getattr(self.ff1, fname)
                self.assertEqual(f1.getFgTValidator(), False)
                f1.setFgTValidator('python:True')
                self.assertEqual(f1.getFgTValidator(), True)

    def testEditAdapter(self):
        for f in self.adapterTypes:
            fname = '{0}1'.format(f)
            self.ff1.invokeFactory(f, fname)
            f1 = getattr(self.ff1, fname)
            f1.setTitle('title')
            f1.setDescription('description')

            self.assertEqual(f1.Title(), 'title')
            self.assertEqual(f1.Description(), 'description')

    def testMailerZPTBody(self):
        fname = 'FormMailerAdapter1'
        self.ff1.invokeFactory('FormMailerAdapter', fname)
        f1 = getattr(self.ff1, fname)
        self.assertTrue(f1.getBody_pt(fields=[], wrappedFields=[]))

    def testEditThanksPages(self):
        for f in self.thanksTypes:
            fname = '{0}1'.format(f)
            self.ff1.invokeFactory(f, fname)
            f1 = getattr(self.ff1, fname)
            f1.setTitle('title')
            f1.setDescription('description')

            self.assertEqual(f1.Title(), 'title')
            self.assertEqual(f1.Description(), 'description')

    def testCreateFieldsAdaptersOutsideEasyForm(self):
        for f in self.fieldTypes + self.adapterTypes + self.thanksTypes + self.fieldsetTypes:
            try:
                self.folder.invokeFactory(f, 'f1')
            except (Unauthorized, ValueError):
                return
            self.fail(
                'Expected error when creating form field or adapter outside form folder.')

    def testBadIdField(self):
        # test for tracker # 32 - Field with id 'language' causes problems with
        # PTS

        from Products.CMFCore.exceptions import BadRequest

        fname = 'test_field'
        self.ff1.invokeFactory('FormStringField', fname)
        f1 = getattr(self.ff1, fname)

        self.assertRaises(BadRequest, f1.setId, 'language')

        # also not such a good idea ...
        self.assertRaises(BadRequest, f1.setId, 'form')

    def testFieldRename(self):
        '''
        renaming a field should change the __name__ attribute
        of the embedded fgField; tracker issue # 42
        '''

        self.ff1.invokeFactory('FormStringField', 'spam_and_eggs')
        self.assertTrue('spam_and_eggs' in self.ff1.objectIds())

        myField = getattr(self.ff1, 'spam_and_eggs')
        fgf = getattr(myField, 'fgField')
        self.assertEqual(fgf.__name__, 'spam_and_eggs')

        # XXX TODO: figure out what's wrong with this:
        # self.ff1.manage_renameObject('spam_and_eggs', 'spam_spam_and_eggs')
        # self.assertEqual(fgf.__name__, 'spam_spam_and_eggs')

    def testFieldsetRename(self):
        '''
        renaming a fieldset should change the __name__ attribute
        of the embedded fsStartField
        '''

        self.ff1.invokeFactory('FieldsetFolder', 'fsfolder1')
        self.assertTrue('fsfolder1' in self.ff1.objectIds())

        myField = getattr(self.ff1, 'fsfolder1')
        fgf = getattr(myField, 'fsStartField')
        self.assertEqual(fgf.__name__, 'fsfolder1')

    def testFieldsetPlusDisplayList(self):
        ''' Test for issue  # 44 -- Presence of fieldset causes an attribute error
        '''

        # create fieldset
        self.ff1.invokeFactory('FieldsetFolder', 'fsf1')
        self.assertTrue(self.ff1.fgFieldsDisplayList())

    def testUtfInFieldTitle(self):
        ''' test for issue # 102, 104: utf8, non-ascii in field title or description
        '''

        self.ff1.invokeFactory('FormStringField', 'sf1',
                               title='Effacer les entr\xc3\xa9es sauvegard\xc3\xa9es'.decode('utf8'))

        self.ff1.sf1.setDescription(
            'Effacer les entr\xc3\xa9es sauvegard\xc3\xa9es'.decode('utf8'))
        # force a reindex
        self.ff1.sf1.reindexObject()

    def testUtfInFormTitle(self):
        ''' test for utf8, non-ascii in form title or description
        '''

        self.folder.invokeFactory('EasyForm', 'ff2',
                                  title='Effacer les entr\xc3\xa9es sauvegard\xc3\xa9es'.decode('utf8'))

        self.folder.ff2.setDescription(
            'Effacer les entr\xc3\xa9es sauvegard\xc3\xa9es'.decode('utf8'))
        # force a reindex
        self.folder.ff2.reindexObject()

    def testBadIds(self):
        ''' test ids that cause problems.
            We shouldn't be able to create objects with ids known
            to be troublesome.
            Also, all fields in all fieldsets must have unique ids.
        '''

        # should be OK:
        self.assertTrue(self.ff1.checkIdAvailable('somethingunique8723'))

        # bad ids should fail
        self.assertFalse(self.ff1.checkIdAvailable('zip'))
        self.assertFalse(self.ff1.checkIdAvailable('location'))
        self.assertFalse(self.ff1.checkIdAvailable('language'))

        # existing ids should fail
        self.ff1.invokeFactory('FormStringField', 'sf1')
        self.assertFalse(self.ff1.checkIdAvailable('sf1'))

        # test in fieldset folder
        self.ff1.invokeFactory('FieldsetFolder', 'fsf1')
        fsf1 = self.ff1.fsf1
        self.assertTrue(fsf1.checkIdAvailable('somethingunique8723'))
        self.assertFalse(fsf1.checkIdAvailable('zip'))

        # We should also not be able to create a fieldset folder field
        # with an id the same as one in the parent form folder
        self.assertFalse(fsf1.checkIdAvailable('sf1'))
        # nor in the fieldset folder itself
        fsf1.invokeFactory('FormStringField', 'sf2')
        self.assertFalse(fsf1.checkIdAvailable('sf2'))

        # Let's try it in a sibling fieldset folder
        self.ff1.invokeFactory('FieldsetFolder', 'fsf2')
        fsf2 = self.ff1.fsf2
        self.assertTrue(fsf2.checkIdAvailable('somethingunique8723'))
        self.assertFalse(fsf2.checkIdAvailable('sf1'))
        self.assertFalse(fsf2.checkIdAvailable('sf2'))


class TestGPG(base.EasyFormTestCase):

    ''' test ya_gpg.py '''

    def test_gpg(self):
        from collective.easyform.content.ya_gpg import gpg, GPGError

        if gpg is None:
            print '\nSkipping GPG tests; gpg binary not found'
        else:
            self.assertRaises(GPGError, gpg.encrypt, 'spam', 'eggs')


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestInstallation))
    # suite.addTest(makeSuite(TestContentCreation))
    # suite.addTest(makeSuite(TestGPG))
    return suite



# -*- coding: utf-8 -*-
#
# Integeration tests specific to the mailer
#

from collective.easyform.api import get_actions
from collective.easyform.api import get_fields
from collective.easyform.api import set_actions
from collective.easyform.api import set_fields
from collective.easyform.interfaces import IActionExtender
from collective.easyform.tests import base
import email


class TestFunctions(base.EasyFormTestCase):

    """ test ya_gpg.py """

    def assertHeader(self, header, addresses, text=None):
        if text is None:
            text = self.messageText
        wanted = '{0}: {1}'.format(header, addresses)
        if wanted in text:
            return
        # Try to get a nicer test failure message.
        lines = [line for line in text.splitlines() if '{0}:'.format(header) in line]
        if not lines:
            # give up
            self.fail('"{0}:" not in message text.'.format(header))
        for line in lines:
            if wanted in line:
                return
        self.fail('{0} not in "{1}:" lines in message text. Found: {2}'.format(
            wanted, header, lines,
        ))

    def assertFrom(self, addresses, text=None):
        return self.assertHeader('From', addresses, text=text)

    def assertSubject(self, addresses, text=None):
        return self.assertHeader('Subject', addresses, text=text)

    def assertTo(self, addresses, text=None):
        return self.assertHeader('To', addresses, text=text)

    def dummy_send(self, mfrom, mto, messageText, immediate=False):
        self.mfrom = mfrom
        self.mto = mto
        self.messageText = messageText
        self.messageBody = '\n\n'.join(messageText.split('\n\n')[1:])

    def afterSetUp(self):
        super(TestFunctions, self).afterSetUp()
        self.folder.invokeFactory('EasyForm', 'ff1')
        self.ff1 = getattr(self.folder, 'ff1')
        self.ff1.CSRFProtection = False  # no csrf protection
        self.mailhost = self.folder.MailHost
        self.mailhost._send = self.dummy_send
        actions = get_actions(self.ff1)
        actions['mailer'].recipient_email = u'mdummy@address.com'
        set_actions(self.ff1, actions)

    def LoadRequestForm(self, **kwargs):
        form = self.app.REQUEST.form
        form.clear()
        for key in kwargs.keys():
            form[key] = kwargs[key]
        return self.app.REQUEST

    def test_DummyMailer(self):
        """ sanity check; make sure dummy mailer works as expected """

        self.mailhost.send(
            'messageText', mto='dummy@address.com', mfrom='dummy1@address.com')
        self.assertTrue(self.messageText.endswith('messageText'))
        self.assertEqual(self.mto, ['dummy@address.com'])
        self.assertTo('dummy@address.com')
        self.assertEqual(self.mfrom, 'dummy1@address.com')
        self.assertFrom('dummy1@address.com')

    def test_Mailer(self):
        """ Test mailer with dummy_send """

        mailer = get_actions(self.ff1)['mailer']

        request = self.LoadRequestForm(
            topic='test subject', comments='test comments')

        mailer.onSuccess(request.form, request)

        self.assertTo('mdummy@address.com')
        self.assertSubject('=?utf-8?q?test_subject?=')
        msg = email.message_from_string(self.messageText)
        self.assertTrue(
            msg.get_payload(decode=True).find('test comments') > 0)

    def test_MailerAdditionalHeaders(self):
        """ Test mailer with dummy_send """

        mailer = get_actions(self.ff1)['mailer']

        request = self.LoadRequestForm(
            topic='test subject', comments='test comments')

        mailer.additional_headers = [
            'Generator: Plone',
            'Token:   abc  ',
        ]

        mailer.onSuccess(request.form, request)

        self.assertTrue(self.messageText.find('Generator: Plone') > 0)
        self.assertTrue(self.messageText.find('Token: abc') > 0)
        self.assertTo('mdummy@address.com')
        self.assertSubject('=?utf-8?q?test_subject?=')
        msg = email.message_from_string(self.messageText)
        self.assertTrue(
            msg.get_payload(decode=True).find('test comments') > 0)

    def test_MailerLongSubject(self):
        """ Test mailer with subject line > 76 chars (Tracker # 84) """

        long_subject = 'Now is the time for all good persons to come to the aid of the quick brown fox.'

        mailer = get_actions(self.ff1)['mailer']
        # fields = self.ff1._getFieldObjects()
        request = self.LoadRequestForm(topic=long_subject)
        mailer.onSuccess(request.form, request)

        msg = email.message_from_string(self.messageText)
        encoded_subject_header = msg['subject']
        decoded_header = email.Header.decode_header(
            encoded_subject_header)[0][0]

        self.assertEqual(decoded_header, long_subject)

    def test_SubjectDollarReplacement(self):
        """
        Simple subject lines should do ${identifier} replacement from request.form --
        but only for a basic override.
        """

        mailer = get_actions(self.ff1)['mailer']
        mailer.msg_subject = 'This is my ${topic} now'

        # baseline unchanged
        request = self.LoadRequestForm(
            topic='test subject', replyto='test@test.org', comments='test comments')
        self.messageText = ''
        mailer.onSuccess(request.form, request)
        self.assertSubject('=?utf-8?q?test_subject?=')

        # no substitution on field replacement (default situation)
        request = self.LoadRequestForm(
            topic='test ${subject}', replyto='test@test.org', comments='test comments')
        self.messageText = ''
        mailer.onSuccess(request.form, request)
        self.assertSubject('=?utf-8?q?test_=24=7Bsubject=7D?=')

        # we should get substitution in a basic override
        mailer.subject_field = ''
        request = self.LoadRequestForm(
            topic='test subject', replyto='test@test.org', comments='test comments')
        self.messageText = ''
        mailer.onSuccess(request.form, request)
        self.assertSubject('=?utf-8?q?This_is_my_test_subject_now?=')

        # we should get substitution in a basic override
        mailer.msg_subject = 'This is my ${untopic} now'
        self.messageText = ''
        mailer.onSuccess(request.form, request)
        self.assertSubject('=?utf-8?q?This_is_my_=3F=3F=3F_now?=')

        # we don't want substitution on user input
        request = self.LoadRequestForm(
            topic='test ${subject}', replyto='test@test.org', comments='test comments')
        self.messageText = ''
        mailer.onSuccess(request.form, request)
        self.assertSubject('=?utf-8?q?This_is_my_=3F=3F=3F_now?=')

    def test_TemplateReplacement(self):
        """
        Mail template prologues, epilogues and footers should do ${identifier}
        replacement from request.form -- this is simpler because there are no
        overrides.
        """

        mailer = get_actions(self.ff1)['mailer']
        mailer.body_pre = 'Hello ${topic},'
        mailer.body_post = 'Thanks, ${topic}!'
        mailer.body_footer = 'Eat my footer, ${topic}.'

        # we should get substitution
        request = self.LoadRequestForm(
            topic='test subject', replyto='test@test.org', comments='test comments')
        self.messageText = ''
        mailer.onSuccess(request.form, request)
        self.assertTrue(self.messageBody.find('Hello test subject,') > 0)
        self.assertTrue(self.messageBody.find('Thanks, test subject!') > 0)
        self.assertTrue(self.messageBody.find(
            'Eat my footer, test subject.') > 0)

    def test_UTF8Subject(self):
        """ Test mailer with uft-8 encoded subject line """

        utf8_subject = 'Effacer les entr\xc3\xa9es sauvegard\xc3\xa9es'

        mailer = get_actions(self.ff1)['mailer']
        # fields = self.ff1._getFieldObjects()
        request = self.LoadRequestForm(topic=utf8_subject)
        mailer.onSuccess(request.form, request)

        msg = email.message_from_string(self.messageText)
        encoded_subject_header = msg['subject']
        decoded_header = email.Header.decode_header(
            encoded_subject_header)[0][0]

        self.assertEqual(decoded_header, utf8_subject)

    def test_UnicodeSubject(self):
        """ Test mailer with Unicode encoded subject line """

        utf8_subject = 'Effacer les entr\xc3\xa9es sauvegard\xc3\xa9es'
        unicode_subject = utf8_subject.decode('UTF-8')

        mailer = get_actions(self.ff1)['mailer']
        # fields = self.ff1._getFieldObjects()
        request = self.LoadRequestForm(topic=unicode_subject)
        mailer.onSuccess(request.form, request)

        msg = email.message_from_string(self.messageText)
        encoded_subject_header = msg['subject']
        decoded_header = email.Header.decode_header(
            encoded_subject_header)[0][0]

        self.assertEqual(decoded_header, utf8_subject)

    def test_MailerOverrides(self):
        """ Test mailer override functions """

        mailer = get_actions(self.ff1)['mailer']
        mailer.subjectOverride = "python: '{0} and {1}'.format('eggs', 'spam')"
        mailer.senderOverride = 'string: spam@eggs.com'
        mailer.recipientOverride = 'string: eggs@spam.com'

        # fields = self.ff1._getFieldObjects()

        request = self.LoadRequestForm(topic='test subject')

        mailer.onSuccess(request.form, request)

        self.assertSubject('=?utf-8?q?eggs_and_spam?=')
        self.assertFrom('spam@eggs.com')
        self.assertTo('eggs@spam.com')

    def test_MailerOverridesWithFieldValues(self):
        mailer = get_actions(self.ff1)['mailer']
        mailer.subjectOverride = 'fields/topic'
        mailer.recipientOverride = 'fields/replyto'

        request = self.LoadRequestForm(
            topic='eggs and spam',
            replyto=u'test@test.ts'
        )
        mailer.onSuccess(request.form, request)

        self.assertSubject('=?utf-8?q?eggs_and_spam?=')
        self.assertTo('test@test.ts')

    def testMultiRecipientOverrideByString(self):
        """ try multiple recipients in recipient override """

        mailer = get_actions(self.ff1)['mailer']
        mailer.recipientOverride = 'string: eggs@spam.com, spam@spam.com'

        # fields = self.ff1._getFieldObjects()

        request = self.LoadRequestForm(topic='test subject')

        mailer.onSuccess(request.form, request)

        self.assertTo('eggs@spam.com, spam@spam.com')

    def testMultiRecipientOverrideByTuple(self):
        """ try multiple recipients in recipient override """

        mailer = get_actions(self.ff1)['mailer']
        mailer.recipientOverride = "python: ('eggs@spam.com', 'spam.spam.com')"

        # fields = self.ff1._getFieldObjects()

        request = self.LoadRequestForm(topic='test subject')

        mailer.onSuccess(request.form, request)

        self.assertTo('eggs@spam.com, spam.spam.com')

    def testRecipientFromRequest(self):
        """ try recipient from designated field  """

        mailer = get_actions(self.ff1)['mailer']
        mailer.to_field = 'replyto'
        mailer.replyto_field = None

        # fields = self.ff1._getFieldObjects()

        request = self.LoadRequestForm(
            topic='test subject', replyto='eggs@spamandeggs.com')

        mailer.onSuccess(request.form, request)

        self.assertTo('eggs@spamandeggs.com')

        request = self.LoadRequestForm(
            topic='test subject', replyto=['eggs@spam.com', 'spam@spam.com'])

        mailer.onSuccess(request.form, request)

        self.assertTo('eggs@spam.com, spam@spam.com')

    def setExecCondition(self, value):
        actions = get_actions(self.ff1)
        IActionExtender(actions['mailer']).execCondition = value
        set_actions(self.ff1, actions)

    def test_ExecConditions(self):
        """ Test mailer with various exec conditions """

        # if an action adapter's execCondition is filled in and evaluates false,
        # the action adapter should not fire.

        view = self.ff1.restrictedTraverse('view')
        form = view.form_instance

        request = self.LoadRequestForm(
            topic='test subject', replyto='test@test.org', comments='test comments')

        self.messageText = ''
        self.setExecCondition('python: False')
        form.processActions(request.form)
        self.assertTrue(len(self.messageText) == 0)

        self.messageText = ''
        self.setExecCondition('python: True')
        form.processActions(request.form)
        self.assertTrue(len(self.messageText) > 0)

        self.messageText = ''
        self.setExecCondition('python: 1==0')
        form.processActions(request.form)
        self.assertTrue(len(self.messageText) == 0)

        # make sure an empty execCondition causes the action to fire
        self.messageText = ''
        self.setExecCondition('')
        form.processActions(request.form)
        self.assertTrue(len(self.messageText) > 0)

    def test_selectiveFieldMailing(self):
        """ Test selective inclusion of fields in the mailing """

        mailer = get_actions(self.ff1)['mailer']
        request = self.LoadRequestForm(
            topic='test subject', replyto='test@test.org', comments='test comments')

        # make sure all fields are sent unless otherwise specified
        self.messageText = ''
        mailer.onSuccess(request.form, request)
        self.assertTrue(
            self.messageBody.find('test subject') > 0 and
            self.messageBody.find('test@test.org') > 0 and
            self.messageBody.find('test comments') > 0
        )

        # setting some show fields shouldn't change that
        mailer.showFields = ('topic', 'comments',)
        self.messageText = ''
        mailer.onSuccess(request.form, request)
        self.assertTrue(
            self.messageBody.find('test subject') > 0 and
            self.messageBody.find('test@test.org') > 0 and
            self.messageBody.find('test comments') > 0
        )

        # until we turn off the showAll flag
        mailer.showAll = False
        self.messageText = ''
        mailer.onSuccess(request.form, request)
        self.assertTrue(
            self.messageBody.find('test subject') > 0 and
            self.messageBody.find('test@test.org') < 0 and
            self.messageBody.find('test comments') > 0
        )

        # check includeEmpties
        mailer.includeEmpties = False

        # first see if everything's still included
        mailer.showAll = True
        self.messageText = ''
        mailer.onSuccess(request.form, request)
        # look for labels
        self.assertTrue(self.messageBody.find('Subject') > 0)
        self.assertTrue(self.messageBody.find('Your E-Mail Address') > 0)
        self.assertTrue(self.messageBody.find('Comments') > 0)

        # now, turn off required for a field and leave it empty
        fields = get_fields(self.ff1)
        fields['comments'].required = False
        set_fields(self.ff1, fields)
        request = self.LoadRequestForm(
            topic='test subject', replyto='test@test.org',)
        self.messageText = ''
        mailer.onSuccess(request.form, request)
        self.assertTrue(self.messageBody.find('Subject') > 0)
        self.assertTrue(self.messageBody.find('Your E-Mail Address') > 0)
        self.assertTrue(self.messageBody.find('Comments') < 0)

    def test_ccOverride(self):
        """ Test override for CC field """

        mailer = get_actions(self.ff1)['mailer']
        request = self.LoadRequestForm(
            topic='test subject', replyto='test@test.org', comments='test comments')

        mailer.cc_recipients = 'test@testme.com'
        self.messageText = ''
        mailer.onSuccess(request.form, request)
        self.assertTrue(
            'test@testme.com' in self.mto
        )

        # simple override
        mailer.ccOverride = 'string:test@testme.com'
        self.messageText = ''
        mailer.onSuccess(request.form, request)
        self.assertTrue(
            'test@testme.com' in self.mto
        )

        # list override
        mailer.ccOverride = "python:['test@testme.com', 'test1@testme.com']"
        self.messageText = ''
        mailer.onSuccess(request.form, request)
        self.assertTrue(
            'test@testme.com' in self.mto and
            'test1@testme.com' in self.mto
        )

    def test_bccOverride(self):
        """ Test override for BCC field """

        mailer = get_actions(self.ff1)['mailer']
        request = self.LoadRequestForm(
            topic='test subject', replyto='test@test.org', comments='test comments')

        mailer.bcc_recipients = 'test@testme.com'
        self.messageText = ''
        mailer.onSuccess(request.form, request)
        self.assertTrue(
            'test@testme.com' in self.mto
        )

        # simple override
        mailer.bccOverride = 'string:test@testme.com'
        self.messageText = ''
        mailer.onSuccess(request.form, request)
        self.assertTrue(
            'test@testme.com' in self.mto
        )

        # list override
        mailer.bccOverride = "python:['test@testme.com', 'test1@testme.com']"
        self.messageText = ''
        mailer.onSuccess(request.form, request)
        self.assertTrue(
            'test@testme.com' in self.mto and
            'test1@testme.com' in self.mto
        )

    def testNoRecipient(self):
        """ try no recipient """

        mailer = get_actions(self.ff1)['mailer']
        mailer.recipient_email = u''
        mailer.to_field = None
        mailer.replyto_field = None

        request = self.LoadRequestForm(
            topic='test subject', replyto='test@test.org', comments='test comments')

        self.messageText = ''
        mailer.onSuccess(request.form, request)
        self.assertTrue(
            'mdummy@address.com' in self.mto
        )


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestFunctions))
    return suite



# -*- coding: utf-8 -*-
from collective.easyform.interfaces import IEasyForm
from plone.dexterity.content import Item
from zope.interface import implements


class EasyForm(Item):
    implements(IEasyForm)



# -*- coding: utf-8 -*-
from collections import namedtuple
from lxml import etree
from Products.PloneFormGen.content.fields import FGFieldsetEnd
from Products.PloneFormGen.content.fields import FGFieldsetStart
from Products.PloneFormGen.content.fieldsBase import BaseFormField
from Products.PloneFormGen.interfaces import IPloneFormGenFieldset

import logging
import six


logger = logging.getLogger('collective.easyform.migration')


NAMESPACES = {
    'easyform': 'http://namespaces.plone.org/supermodel/easyform',
    'form': 'http://namespaces.plone.org/supermodel/form',
}


def append_field(schema, type_, name, properties):
    field = etree.SubElement(schema, u'field')
    field.set(u'name', name)
    field.set(u'type', type_)
    return field


def append_label_field(schema, type_, name, properties):
    field = append_field(schema, type_, name, properties)
    append_node(field, u'required', u'False')
    return field


def append_date_field(schema, type_, name, properties):
    if properties.get('fgShowHM', False):
        return append_field(schema, u'zope.schema.Datetime', name, properties)
    else:
        return append_field(schema, u'zope.schema.Date', name, properties)


def append_fieldset(schema, type_, name, properties):
    fieldset = etree.SubElement(schema, u'fieldset')
    fieldset.set(u'name', name)
    return fieldset


def set_attribute(field, name, value):
    if u':' in name:
        ns, attr = name.split(':')
        ns = NAMESPACES.get(ns, ns)
        field.set(u'{{{}}}{}'.format(ns, attr), value)
    else:
        field.set(name, value)


def append_node(field, name, value):
    if ':' in name:
        ns, name = name.split(':')
        ns = NAMESPACES.get(ns, ns)
        name = '{{{}}}{}'.format(ns, name)
    node = etree.SubElement(field, name)
    if isinstance(value, (list, tuple)):
        value = u' '.join(value)
    node.text = value
    return node


def append_list_node(field, name, value):
    node = append_node(field, name, None)
    for val in value:
        el = etree.SubElement(node, 'element')
        if '|' in val:
            key, val = val.split('|')
            el.set('key', key)
        el.text = val


def append_required_node(field, name, value):
    if value == u'False':
        append_node(field, name, value)


def append_maxlength_node(field, name, value):
    if value != u'0':
        append_node(field, name, value)


def append_vocab_node(field, name, value):
    if field.get('type') == 'zope.schema.Set':
        node = etree.SubElement(field, 'value_type')
        node.set('type', 'zope.schema.Choice')
    else:
        node = field

    append_list_node(node, u'values', value)


def append_default_node(field, name, value):
    if isinstance(value, list):
        return
    if field.get('type') == 'collective.easyform.fields.RichLabel':
        append_node(field, 'rich_label', value)
    else:
        append_node(field, name, value)


def append_widget_node(field, name, value):
    node = append_node(field, name, None)
    type_ = field.get('type')
    if type_ == 'zope.schema.Set':
        if value == 'select':
            widget = 'z3c.form.browser.select.CollectionSelectFieldWidget'
        else:
            widget = 'z3c.form.browser.checkbox.CheckBoxFieldWidget'
    else:
        if value == 'select':
            widget = 'z3c.form.browser.select.ChoiceWidgetDispatcher'
        else:
            widget = 'z3c.form.browser.radio.RadioFieldWidget'
    set_attribute(node, 'type', widget)


def append_or_set_title(field, name, value):
    if field.tag == 'fieldset':
        set_attribute(field, 'label', value)
    else:
        append_node(field, name, value)

def append_or_set_description(field, name, value):
    if field.tag == 'fieldset':
        set_attribute(field, 'description', value)
    else:
        append_node(field, name, value)


def convert_tales_expressions(value):
    if value == u'here/memberEmail':
        return u"python:member and member.getProperty('email', '') or ''"
    elif value == u'here/memberFullName':
        return u"python:member and member.getProperty('fullname', '') or ''"
    elif value == u'here/memberId':
        return u"python:member and member.id or ''"
    return value


def to_text(value):
    if isinstance(value, (list, tuple)):
        return [to_text(v) for v in value]
    value = str(value)
    if six.PY2:
        value = value.decode('utf8')
    return value


Type = namedtuple('Type', ['name', 'handler'])
Property = namedtuple('Property', ['name', 'handler'])

TYPES_MAPPING = {
    'FormStringField': Type('zope.schema.TextLine', append_field),
    'FormPasswordField': Type('zope.schema.Password', append_field),
    'FormIntegerField': Type('zope.schema.Int', append_field),
    'FormFixedPointField': Type('zope.schema.Float', append_field),
    'FormBooleanField': Type('zope.schema.Bool', append_field),
    'FormDateField': Type('zope.schema.Date', append_date_field),
    'FormLabelField': Type('collective.easyform.fields.Label', append_label_field),
    'FormLinesField': Type('zope.schema.Text', append_field),
    'FormSelectionField': Type('zope.schema.Choice', append_field),
    'FormMultiSelectionField': Type('zope.schema.Set', append_field),
    'FormTextField': Type('zope.schema.Text', append_field),
    'FormRichTextField': Type('plone.app.textfield.RichText', append_field),
    'FormRichLabelField': Type('collective.easyform.fields.RichLabel', append_label_field),
    'FormFileField': Type('plone.namedfile.field.NamedBlobFile', append_field),
    'FormCaptchaField': Type('collective.easyform.fields.ReCaptcha', append_field),
    'FieldsetStart': Type('', append_fieldset),
    'FieldsetEnd': Type('', None),
}

PROPERTIES_MAPPING = {
    'description': Property('description', append_or_set_description),
    'fgDefault': Property('default', append_default_node),
    'fgmaxlength': Property('max_length', append_maxlength_node),
    'fgsize': None,  # Not available in collective.easyform
    'fgStringValidator': Property('easyform:validators', set_attribute),
    'fgTDefault': Property('easyform:TDefault', set_attribute),
    'fgTEnabled': Property('easyform:TEnabled', set_attribute),
    'fgTValidator': Property('easyform:TValidator', set_attribute),
    'fgTVocabulary': None,  # Not available in collective.easyform
    'fgVocabulary': Property('values', append_vocab_node),
    'hidden': Property('easyform:THidden', set_attribute),
    'maxval': Property('max', append_node),
    'minval': Property('min', append_node),
    'placeholder': None,  # Not available in collective.easyform
    'required': Property('required', append_required_node),
    'serverSide': Property('easyform:serverSide', set_attribute),
    'title': Property('title', append_or_set_title),
    'fgFormat': Property('form:widget', append_widget_node),
}


def pfg_fields_properties(obj):
    id_ = obj.getId()
    props = {}
    props['_portal_type'] = obj.portal_type
    for field in obj.Schema().fields():
        name = field.getName()
        accessor = field.getEditAccessor(obj)
        props[name] = accessor()
    return (id_, props)


def pfg_fields(context):
    fields = []
    for obj in context.objectValues():
        if IPloneFormGenFieldset.providedBy(obj):
            # handle FieldsetFolder folderish type
            parent = obj.aq_parent
            obj_id = obj.getId()
            # append a FieldsetStart field with title and description
            start = FGFieldsetStart(obj_id).__of__(parent)
            start.setTitle(obj.Title())
            start.setDescription(obj.Description())
            fields.append(pfg_fields_properties(start))
            # add all the fieldset contained fields
            fields.extend(pfg_fields(obj))
            # finish by appending a marker FieldsetEnd field
            end = FGFieldsetEnd(obj_id).__of__(parent)
            fields.append(pfg_fields_properties(end))
            continue
        if not isinstance(obj, BaseFormField):
            continue
        fields.append(pfg_fields_properties(obj))
    return fields


def fields_model(ploneformgen):
    """Create a fields xml model from a PloneFormGen instance."""
    parser = etree.XMLParser(remove_blank_text=True)
    model = etree.fromstring(FIELDS_MODEL, parser)
    schema = model.find(
        '{http://namespaces.plone.org/supermodel/schema}schema')
    for fieldname, properties in pfg_fields(ploneformgen):
        portal_type = properties['_portal_type']
        if portal_type == 'FieldsetEnd':
            schema = schema.getparent()
            continue

        type_ = TYPES_MAPPING.get(portal_type)
        if type_ is None:
            # ploneformgen has been renamed with _MIGRATION_ suffix
            # hence, we remove 11 chars
            pfgpath = "/".join(ploneformgen.getPhysicalPath())[:-11]
            logger.warning(
                "Ignoring field '%s' of type '%s' of form '%s'.", fieldname, portal_type, pfgpath)
            continue

        if type_.handler is None:
            continue
        field = type_.handler(schema, type_.name, fieldname, properties)

        for name, value in properties.items():
            prop = PROPERTIES_MAPPING.get(name)
            if prop is None:
                continue

            value = to_text(value)

            # Convert TALES expressions with PFG specific methods
            if name.startswith('fgT'):
                value = convert_tales_expressions(value)

            prop.handler(field, prop.name, value)

        if portal_type == 'FieldsetStart':
            schema = field

    return etree.tostring(model, pretty_print=True)


FIELDS_MODEL = b"""
<model
    xmlns="http://namespaces.plone.org/supermodel/schema"
    xmlns:easyform="http://namespaces.plone.org/supermodel/easyform"
    xmlns:i18n="http://xml.zope.org/namespaces/i18n"
    i18n:domain="collective.easyform">
  <schema></schema>
</model>
"""



# -*- coding: utf-8 -*-

from Products.Five import BrowserView
from Products.GenericSetup.context import TarballExportContext
from Products.GenericSetup.context import TarballImportContext
from Products.GenericSetup.interfaces import IFilesystemExporter
from Products.GenericSetup.interfaces import IFilesystemImporter
from Products.statusmessages.interfaces import IStatusMessage
from collective.easyform import easyformMessageFactory as _
from collective.easyform.interfaces import IEasyFormImportFormSchema
from datetime import datetime
from plone.z3cform import layout
from z3c.form import button
from z3c.form import field
from z3c.form import form
from zope.component import getMultiAdapter


class EasyFormExportView(BrowserView):

    """See ..interfaces.exportimport.IEasyFormExportView
    """

    def __call__(self):
        """See ..interfaces.exportimport.IEasyFormExportView.__call__
        """
        ctx = TarballExportContext(self.context)
        response = self.request.RESPONSE
        disposition = 'attachment; filename="{0}-{1:{2}}.tar.gz"'.format(
            self.context.getId(), datetime.now(), '%Y%m%d%H%M%S')

        response.setHeader('Content-type', 'application/x-gzip')
        response.setHeader('Content-disposition', disposition)

        # export the structure treating the current form as our root context
        IFilesystemExporter(self.context).export(ctx, 'structure', True)

        return ctx.getArchive()


class EasyFormImportForm(form.Form):

    """The form class for importing of exported easyforms
    """
    fields = field.Fields(IEasyFormImportFormSchema)
    ignoreContext = True
    ignoreReadonly = True

    @button.buttonAndHandler(_(u'import'), name='import')
    def handleImport(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        ctx = TarballImportContext(self.context, data['upload'])
        IFilesystemImporter(self.context).import_(ctx, 'structure', True)

        self.status = _(u'Form imported.')
        IStatusMessage(self.request).addStatusMessage(self.status, type='info')

        url = getMultiAdapter(
            (self.context, self.request), name='absolute_url')()
        self.request.response.redirect(url)


EasyFormImportView = layout.wrap_form(EasyFormImportForm)



# -*- coding: utf-8 -*-
#
# Test EasyForm top-level functionality
#

from Products.CMFCore.utils import getToolByName
from collective.easyform.tests import base


class FakeRequest(dict):

    def __init__(self, **kwargs):
        self.form = kwargs


class TestTools(base.EasyFormTestCase):

    """ test our tool """

    def test_FormGenTool(self):
        fgt = getToolByName(self.portal, 'formgen_tool')

        # pt = getToolByName(
        #    self.portal, 'portal_properties').easyform_properties

        fgt.setDefault('permissions_used', ['test text'])
        fgt.setDefault('mail_template', 'something')
        fgt.setDefault('mail_body_type', 'text')
        fgt.setDefault('mail_recipient_email', 'eggs')
        fgt.setDefault('mail_recipient_name', 'spam')
        fgt.setDefault('mail_cc_recipients', ['spam and eggs'])
        fgt.setDefault('mail_bcc_recipients', ['eggs and spam'])
        fgt.setDefault('mail_xinfo_headers', ['one', 'two'])
        fgt.setDefault('mail_add_headers', ['three', 'four'])
        fgt.setDefault('csv_delimiter', '|')

        permits = fgt.getPfgPermissions()
        self.assertTrueEqual(len(permits), 1)
        self.assertTrueEqual(permits[0], 'test text')

        self.assertTrueEqual(fgt.getDefaultMailTemplateBody(), 'something')
        self.assertTrueEqual(fgt.getDefaultMailBodyType(), 'text')
        self.assertTrueEqual(fgt.getDefaultMailRecipient(), 'eggs')
        self.assertTrueEqual(fgt.getDefaultMailRecipientName(), 'spam')
        self.assertTrueEqual(fgt.getCSVDelimiter(), '|')

        cc = fgt.getDefaultMailCC()
        self.assertTrueEqual(len(cc), 1)
        self.assertTrueEqual(cc[0], 'spam and eggs')

        bcc = fgt.getDefaultMailBCC()
        self.assertTrueEqual(len(bcc), 1)
        self.assertTrueEqual(bcc[0], 'eggs and spam')

        xi = fgt.getDefaultMailXInfo()
        self.assertTrueEqual(len(xi), 2)
        self.assertTrueEqual(xi[0], 'one')

        xi = fgt.getDefaultMailAddHdrs()
        self.assertTrueEqual(len(xi), 2)
        self.assertTrueEqual(xi[0], 'three')

    def test_toolRolesForPermission(self):
        fgt = getToolByName(self.portal, 'formgen_tool')

        # make sure rolesForPermission works
        roleList = fgt.rolesForPermission('EasyForm: Add Content')
        self.assertNotEqual(len(roleList), 0)
        mid = ''
        oid = ''
        for role in roleList:
            if role['label'] == 'Manager':
                self.assertTrueEqual(role['checked'], 'CHECKED')
                mid = role['id']
            if role['label'] == 'Owner':
                self.assertTrueEqual(role['checked'], 'CHECKED')
                oid = role['id']
        self.assertTrue(mid)
        self.assertTrue(oid)

        # let's try changing a permission

        # first, get the request ids
        roleList = fgt.rolesForPermission('EasyForm: Edit Advanced Fields')
        self.assertNotEqual(len(roleList), 0)
        mid = ''
        oid = ''
        for role in roleList:
            if role['label'] == 'Manager':
                self.assertTrueEqual(role['checked'], 'CHECKED')
                mid = role['id']
            if role['label'] == 'Owner':
                self.assertTrueEqual(role['checked'], None)
                oid = role['id']
        self.assertTrue(mid)
        self.assertTrue(oid)

        fr = FakeRequest()
        fr.form[mid] = '1'
        fr.form[oid] = '1'
        fr.form['EasyForm: Edit Advanced Fields'] = '1'
        fgt.setRolePermits(fr)

        # now, check to see if it took
        roleList = fgt.rolesForPermission('EasyForm: Edit Advanced Fields')
        self.assertNotEqual(len(roleList), 0)
        mid = ''
        oid = ''
        for role in roleList:
            if role['label'] == 'Manager':
                self.assertTrueEqual(role['checked'], 'CHECKED')
                mid = role['id']
            if role['label'] == 'Owner':
                self.assertTrueEqual(role['checked'], 'CHECKED')
                oid = role['id']
        self.assertTrue(mid)
        self.assertTrue(oid)


def test_suite():
    from unittest import TestSuite  # , makeSuite
    suite = TestSuite()
    # suite.addTest(makeSuite(TestTools))
    return suite



# -*- coding: utf-8 -*-
from collections import namedtuple
from collective.easyform.migration.fields import append_field
from collective.easyform.migration.fields import append_list_node
from collective.easyform.migration.fields import append_node
from collective.easyform.migration.fields import set_attribute
from collective.easyform.migration.fields import to_text
from lxml import etree
from Products.PloneFormGen.content.actionAdapter import FormActionAdapter

import logging

try:
  from collective.easyform.interfaces.mailer import default_mail_body
except ImportError:
  from collective.easyform.config import MAIL_BODY_DEFAULT
  def default_mail_body():
    return MAIL_BODY_DEFAULT


logger = logging.getLogger('collective.easyform.migration')


NAMESPACES = {
    'easyform': 'http://namespaces.plone.org/supermodel/easyform',
    'form': 'http://namespaces.plone.org/supermodel/form',
}


def append_body_pt(field, name, value):
    node = append_node(field, name, value)
    # Replace mailer page template with the default one from easyform as it's
    # unlikely that the PFG template would work.
    node.text = default_mail_body()


Type = namedtuple('Type', ['name', 'handler'])
Property = namedtuple('Property', ['name', 'handler'])


TYPES_MAPPING = {
    'FormMailerAdapter': Type('collective.easyform.actions.Mailer', append_field),
    'FormSaveDataAdapter': Type('collective.easyform.actions.SaveData', append_field),
    'FormCustomScriptAdapter': Type('collective.easyform.actions.CustomScript', append_field),
}

PROPERTIES_MAPPING = {
    'additional_headers': Property('additional_headers', append_node),
    'bcc_recipients': Property('bcc_recipients', append_node),
    'bccOverride': Property('bccOverride', append_node),
    'body_footer': Property('body_footer', append_node),
    'body_post': Property('body_post', append_node),
    'body_pre': Property('body_pre', append_node),
    'body_pt': Property('body_pt', append_body_pt),
    'body_type': Property('body_type', append_node),
    'cc_recipients': Property('cc_recipients', append_node),
    'ccOverride': Property('ccOverride', append_node),
    'description': Property('description', append_node),
    'execCondition': Property('easyform:execCondition', set_attribute),
    'msg_subject': Property('msg_subject', append_node),
    'recipient_email': Property('recipient_email', append_node),
    'recipient_name': Property('recipient_name', append_node),
    'recipientOverride': Property('recipientOverride', append_node),
    'replyto_field': Property('replyto_field', append_node),
    'senderOverride': Property('senderOverride', append_node),
    'showAll': Property('showAll', append_node),
    'showFields': Property('showFields', append_list_node),
    'subject_field': Property('subject_field', append_node),
    'subjectOverride': Property('subjectOverride', append_node),
    'title': Property('title', append_node),
    'to_field': Property('to_field', append_node),
    'xinfo_headers': Property('xinfo_headers', append_node),
    'ExtraData': Property('ExtraData', append_list_node),
    'DownloadFormat': Property('DownloadFormat', append_node),
    'UseColumnNames': Property('UseColumnNames', append_node),
    'SavedFormInput': Property('SavedFormInput', append_node),
    'ProxyRole': Property('ProxyRole', append_node),
    'ScriptBody': Property('ScriptBody', append_node),
}


def pfg_actions(context):
    actions = []
    for obj in context.objectValues():
        if not isinstance(obj, FormActionAdapter):
            continue
        id_ = obj.getId()
        props = {}
        props['_portal_type'] = obj.portal_type
        for field in obj.Schema().fields():
            name = field.getName()
            accessor = field.getEditAccessor(obj)
            props[name] = accessor()
        actions.append((id_, props))
    return actions


def actions_model(ploneformgen):
    """Create an actions xml model from a PloneFormGen instance."""
    parser = etree.XMLParser(remove_blank_text=True)
    model = etree.fromstring(ACTIONS_MODEL, parser)
    schema = model.find(
        '{http://namespaces.plone.org/supermodel/schema}schema')

    for actionname, properties in pfg_actions(ploneformgen):
        type_ = TYPES_MAPPING.get(properties['_portal_type'])
        if type_ is None:
            # ploneformgen has been renamed with _MIGRATION_ suffix
            # hence, we remove 11 chars
            pfgpath = "/".join(ploneformgen.getPhysicalPath())[:-11]
            logger.warning(
                "Ignoring field '%s' of type '%s' of form '%s.",
                actionname, properties['_portal_type'], pfgpath)
            continue

        field = type_.handler(schema, type_.name, actionname, properties)

        for name, value in properties.items():
            prop = PROPERTIES_MAPPING.get(name)
            if prop is None:
                continue

            value = to_text(value)

            prop.handler(field, prop.name, value)

    return etree.tostring(model, pretty_print=True)


ACTIONS_MODEL = b"""
<model xmlns="http://namespaces.plone.org/supermodel/schema">
  <schema></schema>
</model>
"""



# -*- coding: utf-8 -*-
from collective.easyform.tests import base
from os import getenv
from os import listdir
from os import path
from plone.testing import layered
from robotsuite import RobotTestSuite
from unittest import TestSuite


RUN_ROBOT_TESTS_ENV = "RUN_ROBOT_TESTS"
try:
    RUN_ROBOT_TESTS = int(getenv(RUN_ROBOT_TESTS_ENV, 0))
except (ValueError, TypeError, AttributeError):
    print("Ignored non-integer {0} environment variable.".format(RUN_ROBOT_TESTS_ENV))
    RUN_ROBOT_TESTS = False


def test_suite():
    suite = TestSuite()
    if not RUN_ROBOT_TESTS:
        print(
            "WARNING: Ignoring robot tests because {0} environment variable is not set.".format(
                RUN_ROBOT_TESTS_ENV
            )
        )
        print(
            "Half of the robot tests would fail because modals are not working. Fixing the modals would be appreciated."
        )
        return suite
    current_dir = path.abspath(path.dirname(__file__))
    robot_dir = path.join(current_dir, "robot")
    robot_tests = [
        path.join("robot", doc)
        for doc in listdir(robot_dir)
        if doc.startswith("test") and doc.endswith(".robot")
    ]
    for test in robot_tests:
        suite.addTests([layered(RobotTestSuite(test), layer=base.ROBOT_TESTING)])
    return suite



# -*- coding: utf-8 -*-
'''

    Unit test for EasyForm custom scripts

    Copyright 2006 Red Innovation http://www.redinnovation.com

'''
__author__ = 'Mikko Ohtamaa <mikko@redinnovation.com>'
__docformat__ = 'plaintext'

try:
    from App.class_init import InitializeClass
except ImportError:
    from Globals import InitializeClass

from AccessControl import ClassSecurityInfo
from AccessControl import Unauthorized
from Products.CMFCore import permissions
from collective.easyform.api import get_actions
from collective.easyform.api import set_actions
from collective.easyform.tests import base
from plone.app.testing import logout
from unittest import TestSuite
from unittest import makeSuite


test_script = '''
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=fields, easyform, request
##title=Succesfully working script
##

from Products.CMFCore.utils import getToolByName

assert fields['test_field'] == '123'
return 'foo'
'''

bad_parameters_script = '''
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=
##title=
##

return 'foo'
'''

syntax_error_script = '''
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=fields, easyform, request
##title=
##
if:
return asdfaf
'''

runtime_error_script = '''
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=fields, easyform
##title=
##
return 'asdfaf' + 1
'''

security_script = '''
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=fields, easyform, request
##title=Script needing special privileges
##

from Products.CMFCore.utils import getToolByName

portal_url = getToolByName(context, 'portal_url')
portal = portal_url.getPortalObject()

print portal

# Try set left_slots
portal.manage_addProperty('foo', ['foo'], 'lines')
'''

proxied_script = '''
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=fields, easyform, request
##title=Script needing special privileges
##

# Should raise Unauthorized
return request.fooProtected()
'''

return_error_script = '''
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=fields, easyform, request
##title=
##
return {'comments': 'Please enter more text'}
'''

default_params_script = '''
fields
easyform
request
'''


class FakeRequest(dict):

    def __init__(self, **kwargs):
        self.form = kwargs


class SecureFakeRequest(dict):

    security = ClassSecurityInfo()

    def __init__(self, **kwargs):
        self.form = kwargs

    security.declareProtected(permissions.ManagePortal,
                              'fooProtected')

    def fooProtected(self):
        ''' Only manager can access this '''
        return 'foo'

InitializeClass(SecureFakeRequest)


class TestCustomScript(base.EasyFormTestCase):

    ''' Test FormCustomScriptAdapter functionality in EasyForm '''

    def afterSetUp(self):
        super(TestCustomScript, self).afterSetUp()

        # self.loginAsPortalOwner()

        self.folder.invokeFactory('EasyForm', 'ff1')
        self.ff1 = getattr(self.folder, 'ff1')
        self.ff1.CSRFProtection = False
        self.portal.REQUEST['form.widgets.title'] = u'Test field'
        self.portal.REQUEST['form.widgets.__name__'] = u'test_field'
        self.portal.REQUEST['form.widgets.description'] = u''
        # self.portal.REQUEST['form.widgets.factory'] = ['Text line (String)']
        self.portal.REQUEST['form.widgets.factory'] = ['label_textline_field']
        self.portal.REQUEST['form.widgets.required'] = []
        self.portal.REQUEST['form.buttons.add'] = u'Add'
        view = self.ff1.restrictedTraverse('fields/@@add-field')
        view.update()
        form = view.form_instance
        # form.update()
        data, errors = form.extractData()
        self.assertEqual(len(errors), 0)

    def createScript(self):
        ''' Creates FormCustomScript object '''
        # 1. Create custom script adapter in the form folder
        self.portal.REQUEST['form.widgets.title'] = u'Adapter'
        self.portal.REQUEST['form.widgets.__name__'] = u'adapter'
        self.portal.REQUEST['form.widgets.description'] = u''
        self.portal.REQUEST['form.widgets.factory'] = ['Custom Script']
        self.portal.REQUEST['form.buttons.add'] = u'Add'
        view = self.ff1.restrictedTraverse('actions/@@add-action')
        view.update()
        form = view.form_instance
        data, errors = form.extractData()
        self.assertEqual(len(errors), 0)

        # 2. Check that creation succeeded
        actions = get_actions(self.ff1)
        self.assertTrue('adapter' in actions)

#    def testScriptTypes(self):
#        ''' Check DisplayList doesn't fire exceptions '''
#        self.createScript()
#        adapter = self.ff1.adapter
#        adapter.getScriptTypeChoices()

    def testReturnError(self):
        ''' Succesful script execution with return error
        '''
        self.createScript()

        actions = get_actions(self.ff1)
        actions['adapter'].ScriptBody = return_error_script
        set_actions(self.ff1, actions)

        self.portal.REQUEST['form.widgets.test_field'] = u'Test field'
        self.portal.REQUEST['form.widgets.topic'] = u'subject'
        self.portal.REQUEST['form.widgets.comments'] = u'some comments'
        self.portal.REQUEST['form.widgets.replyto'] = u'foobar@example.com'
        self.portal.REQUEST['form.buttons.submit'] = u'Submit'

        view = self.ff1.restrictedTraverse('view')
        form = view.form_instance
        form.update()

        errors = form.widgets.errors
        self.assertEqual(len(errors), 1)
        self.assertEqual(errors[0].message, 'Please enter more text')

        data, errors = form.extractData()
        self.assertEqual(len(errors), 0)

        errors = form.processActions(data)
        self.assertEqual(errors, {'comments': 'Please enter more text'})

    def testSuccess(self):
        ''' Succesful script execution

        Creates a script, some form content,
        executes form handling.
        '''

        self.createScript()

        actions = get_actions(self.ff1)
        adapter = actions['adapter']

        # 4. Set script data
        adapter.ScriptBody = test_script

        req = FakeRequest(test_field='123')

        reply = adapter.onSuccess({}, req)
        assert reply == 'foo', 'Script returned:' + str(reply)

    def testRunTimeError(self):
        ''' Script has run-time error '''
        self.createScript()

        actions = get_actions(self.ff1)
        adapter = actions['adapter']

        # 4. Set script data
        adapter.ScriptBody = runtime_error_script

        # Execute script
        throwed = False
        try:
            reply = adapter.onSuccess({})
        except TypeError:
            reply = None
            throwed = True

        assert throwed, "Bad script didn't throw run-time exception, got " + \
            str(reply)
        assert reply is None

    def testSyntaxError(self):
        ''' Script has syntax errors

        TODO: Syntax errors are not returned in validation?
        '''

        # Note: this test logs an error message; it does not indicate test
        # failure

        self.createScript()

        actions = get_actions(self.ff1)
        adapter = actions['adapter']

        # 4. Set script data
        adapter.ScriptBody = syntax_error_script

        # Execute script
        throwed = False
        try:
            adapter.onSuccess({}, FakeRequest())
        except ValueError:
            throwed = True

        assert throwed, "Bad script didn't throw run-time exception"

    def testBadParameters(self):
        ''' Invalid number of script parameters '''

        self.createScript()

        actions = get_actions(self.ff1)
        adapter = actions['adapter']

        # 4. Set script data
        adapter.ScriptBody = bad_parameters_script

        # Execute script
        throwed = False
        try:
            adapter.onSuccess([])
        except TypeError:
            throwed = True
        assert throwed, 'Invalid parameters failed silently'

    def testDefaultParameters(self):
        ''' Test to make sure the documented parameters are available '''

        self.createScript()

        actions = get_actions(self.ff1)
        adapter = actions['adapter']

        # 4. Set script data
        adapter.ScriptBody = default_params_script

        request = FakeRequest(
            topic='test subject', replyto='test@test.org', comments='test comments')

        errors = adapter.onSuccess({}, request)
        self.assertEqual(errors, None)

    def testSecurity(self):
        ''' Script needing proxy role

        TODO: Why no security exceptions are raised?
        '''
        self.createScript()
        logout()

        actions = get_actions(self.ff1)
        adapter = actions['adapter']

        # 4. Set script data
        adapter.ScriptBody = security_script

        # errors = adapter.validate()
        # assert len(errors) == 0, 'Had errors:' + str(errors)

        # Execute script
        throwed = False
        try:
            adapter.onSuccess({}, FakeRequest())
        except Unauthorized:
            throwed = True

        if self.portal.hasProperty('foo'):
            assert 'Script executed under full priviledges'

        self.assertTrue(throwed, 'Bypassed security, baaad!')

        adapter.ProxyRole = u'Manager'
        throwed = False
        try:
            adapter.onSuccess({}, FakeRequest())
        except Unauthorized:
            throwed = True

        if not self.portal.hasProperty('foo'):
            assert 'Script not executed thru proxy role'
        self.assertFalse(throwed, 'Unauthorized was raised!')

    def testSetProxyRole(self):
        ''' Exercise setProxyRole '''

        self.createScript()
        self.portal.REQUEST['form.widgets.title'] = u'Adapter'
        self.portal.REQUEST['form.widgets.description'] = u''
        self.portal.REQUEST['form.widgets.ProxyRole'] = [u'Manager']
        self.portal.REQUEST[
            'form.widgets.ScriptBody'] = unicode(proxied_script)
        self.portal.REQUEST['form.widgets.IActionExtender.execCondition'] = u''
        self.portal.REQUEST['form.buttons.save'] = u'Save'
        view = self.ff1.restrictedTraverse('actions')
        view = view.publishTraverse(view.request, 'adapter')
        view = view.publishTraverse(view.request, 'adapter')
        view.update()
        form = view.form_instance
        data, errors = form.extractData()
        self.assertEqual(len(errors), 0)
        self.portal.REQUEST['form.widgets.title'] = u'Adapter'
        self.portal.REQUEST['form.widgets.description'] = u''
        self.portal.REQUEST['form.widgets.ProxyRole'] = [u'none']
        self.portal.REQUEST[
            'form.widgets.ScriptBody'] = unicode(proxied_script)
        self.portal.REQUEST['form.widgets.IActionExtender.execCondition'] = u''
        self.portal.REQUEST['form.buttons.save'] = u'Save'
        view = self.ff1.restrictedTraverse('actions')
        view = view.publishTraverse(view.request, 'adapter')
        view = view.publishTraverse(view.request, 'adapter')
        view.update()
        form = view.form_instance
        data, errors = form.extractData()
        self.assertEqual(len(errors), 0)
        self.portal.REQUEST['form.widgets.title'] = u'Adapter'
        self.portal.REQUEST['form.widgets.description'] = u''
        self.portal.REQUEST['form.widgets.ProxyRole'] = [u'bogus']
        self.portal.REQUEST[
            'form.widgets.ScriptBody'] = unicode(proxied_script)
        self.portal.REQUEST['form.widgets.IActionExtender.execCondition'] = u''
        self.portal.REQUEST['form.buttons.save'] = u'Save'
        view = self.ff1.restrictedTraverse('actions')
        view = view.publishTraverse(view.request, 'adapter')
        view = view.publishTraverse(view.request, 'adapter')
        view.update()
        form = view.form_instance
        data, errors = form.extractData()
        self.assertEqual(len(errors), 1)
        self.assertEqual(errors[0].message, u'Required input is missing.')

    # XXX TODO: We need to find another way to test this.
    def testProxyRole(self):
        ''' Test seeing how setting proxy role affects unauthorized exception '''

        # TODO: Zope security system kills me
        self.createScript()

        actions = get_actions(self.ff1)
        adapter = actions['adapter']

        # 4. Set script data
        adapter.ScriptBody = proxied_script

        req = SecureFakeRequest(test_field='123')

        # errors = adapter.validate()
        # assert len(errors) == 0, 'Had errors:' + str(errors)

        # Execute script
        throwed = False
        try:
            adapter.onSuccess({}, req)
        except Unauthorized:
            throwed = True

        assert throwed, 'No Unauthorized was raised'

#    def testSkinsScript(self):
#        ''' Test executing script from portal_skins '''
#        portal_skins = self.portal.portal_skins
#        manage_addPythonScript(portal_skins.custom, 'test_skins_script')
#        test_skins_script = portal_skins.custom.test_skins_script
#
#        test_skins_script.ZPythonScript_edit('', test_script)
#        self._refreshSkinData()
#
#
#        portal_skins.custom.test_skins_script({'test_field' : '123'}, 'foo', None)
# Do a dummy test call
#        self.portal.test_skins_script({'test_field' : '123'}, 'foo', None)
#
#        self.createScript()
#        adapter = self.ff1.adapter
#        adapter.setScriptType('skins_script')
#        adapter.setScriptName('test_skins_script')
#
#        errors = adapter.validate()
#        assert len(errors) == 0, 'Had errors:' + str(errors)
#
# Execute script
#        req = FakeRequest(test_field='123')
#        reply = adapter.onSuccess([], req)
#        assert reply == 'foo', 'Script returned:' + str(reply)


def test_suite():
    suite = TestSuite()
    suite.addTest(makeSuite(TestCustomScript))
    return suite



# -*- coding: utf-8 -*-

from collective.easyform import easyformMessageFactory as _
from collective.easyform.api import get_context
from collective.easyform.api import get_fields
from z3c.form.interfaces import IFieldWidget
from zope.component import getGlobalSiteManager
from zope.component import getUtilitiesFor
from zope.i18n import translate
from zope.i18nmessageid import MessageFactory
from zope.interface import directlyProvides
from zope.interface import implements
from zope.schema import getFieldsInOrder
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.interfaces import IVocabulary
from zope.schema.vocabulary import SimpleVocabulary


PMF = MessageFactory('plone')

_make_vocabulary = lambda items: SimpleVocabulary([
    SimpleVocabulary.createTerm(token, token, name)
    for (name, token) in items
])

customActions = _make_vocabulary((
    (_(u'Traverse to'), u'traverse_to'),
    (_(u'Redirect to'), u'redirect_to'),
))

MIME_LIST = _make_vocabulary((
    (u'HTML', u'html'),
    (PMF(u'Text'), u'plain'),
))

FORM_METHODS = SimpleVocabulary.fromValues((
    u'post',
    u'get',
))

XINFO_HEADERS = SimpleVocabulary.fromValues((
    u'HTTP_X_FORWARDED_FOR',
    u'REMOTE_ADDR',
    u'PATH_INFO',
    u'HTTP_USER_AGENT',
    u'HTTP_REFERER',
))

getProxyRoleChoices = _make_vocabulary((
    (u'No proxy role', u'none'),
    (u'Manager', u'Manager'),
))

vocabExtraDataDL = _make_vocabulary((
    (_(u'vocabulary_postingdt_text', default=u'Posting Date/Time'), u'dt'),
    (u'HTTP_X_FORWARDED_FOR', u'HTTP_X_FORWARDED_FOR'),
    (u'REMOTE_ADDR', u'REMOTE_ADDR'),
    (u'HTTP_USER_AGENT', u'HTTP_USER_AGENT'),
))

vocabFormatDL = _make_vocabulary((
    (_(u'vocabulary_tsv_text', default=u'Tab-Separated Values'), u'tsv'),
    (_(u'vocabulary_csv_text', default=u'Comma-Separated Values'), u'csv'),
))


class fields(object):

    """Context source binder to provide a vocabulary of users in a given
    group.
    """

    implements(IContextSourceBinder, IVocabulary)

    def __contains__(self, value):
        return True

    def __call__(self, context):
        terms = []
        if hasattr(context, 'interface'):
            form = get_context(context)
        elif hasattr(context, 'fields_model'):
            form = context
        else:
            return SimpleVocabulary(terms)
        fields = getFieldsInOrder(get_fields(form))
        for name, field in fields:
            terms.append(
                SimpleVocabulary.createTerm(name, str(name), field.title))
        return SimpleVocabulary(terms)

fieldsFactory = fields()


class WidgetVocabulary(SimpleVocabulary):

    def getTerm(self, value):
        """See zope.schema.interfaces.IBaseVocabulary"""
        if not isinstance(value, basestring):
            value = '{0}.{1}'.format(
                value.widget_factory.__module__, value.widget_factory.__name__)
        return self.getTermByToken(value)


def widgetsFactory(context):
    terms = []
    adapters = [
        a.factory
        for a in getGlobalSiteManager().registeredAdapters()
        if a.provided == IFieldWidget and len(a.required) == 2 and a.required[0].providedBy(context)
    ]
    for adapter in set(adapters):
        name = u'{0}.{1}'.format(
            adapter.__module__, adapter.__name__)
        terms.append(WidgetVocabulary.createTerm(
            name, str(name), adapter.__name__))
    return WidgetVocabulary(terms)

directlyProvides(widgetsFactory, IContextSourceBinder)


def EasyFormActionsVocabularyFactory(context):
    """EasyForm actions vocabulary"""
    from collective.easyform.interfaces import IActionFactory
    return SimpleVocabulary([
        SimpleVocabulary.createTerm(
            factory, translate(factory.title), factory.title)
        for (id, factory) in getUtilitiesFor(IActionFactory)
        if factory.available(context)
    ])


def ValidatorsVocabularyFactory(context):
    """Field validators vocabulary"""
    from collective.easyform.interfaces import IFieldValidator
    return SimpleVocabulary([
        SimpleVocabulary.createTerm(i, i, i)
        for i, u in getUtilitiesFor(IFieldValidator)
    ])



# -*- coding: utf-8 -*-
#
# Integration tests. See other test modules for specific components.
#

from Testing.makerequest import makerequest
from ZPublisher.BaseRequest import BaseRequest
from collective.easyform.api import get_fields
from collective.easyform.tests import base
from plone.protect.authenticator import AuthenticatorView
from z3c.form.interfaces import IFormLayer
from zope.component import getMultiAdapter
from zope.interface import classImplements
import plone.protect
import sys
import zExceptions


# too lazy to see if this is already in the library somewhere
def stripWhiteSpace(multiLineString):
    return '\n'.join([s.strip() for s in multiLineString.split('\n')])


class TestFunctions(base.EasyFormTestCase):

    """ tests that mostly concern functionality beyond the unit """

    def dummy_send(self, mfrom, mto, messageText, immediate=False):
        self.mfrom = mfrom
        self.mto = mto
        self.messageText = messageText

    def fakeRequest(self, **kwargs):
        self.request.form.clear()
        self.request._authenticator = plone.protect.createToken()
        self.request.form.update(kwargs)
        self.request.form['_authenticator'] = self.request._authenticator
        return self.request

    def afterSetUp(self):
        super(TestFunctions, self).afterSetUp()
        self.folder.invokeFactory('EasyForm', 'ff1')
        self.ff1 = getattr(self.folder, 'ff1')
        self.ff1.title = u'ff1'
        self.mailhost = self.folder.MailHost
        self.mailhost._send = self.dummy_send
        self.request = makerequest(self.app).REQUEST
        self.ff1.CSRFProtection = False
        self.ff1.actions_model = (
            self.ff1.actions_model.replace(
                u'<description>E-Mails Form Input</description>',
                u'<recipient_email>mdummy@address.com</recipient_email><description>E-Mails Form Input</description>'))
        self.mailhost = self.folder.MailHost
        self.mailhost._send = self.dummy_send
        classImplements(BaseRequest, IFormLayer)

    def testFgFieldsDisplayList(self):
        """ test EasyForm's fgFieldsDisplayList """

        # in v 1.0.2, this caused "'unicode' object has no attribute 'decode'"
        # w/ Plone 2.5.1 and Zope 2.8.7
        fields = get_fields(self.ff1)
        res = dict([(i, fields[i]) for i in fields])

        self.assertEqual(len(res), 3)
        self.assertEqual(res.keys()[0], 'replyto')
        self.assertEqual(res.values()[0]._type, unicode)

    def testFgFieldsDisplayListFieldset(self):
        """ Make sure fgFieldsDisplayList works for fields in fieldsets.
            Tracker # 123.
        """

        self.ff1.invokeFactory('FieldsetFolder', 'fsf1', title='a fieldset')
        fsf1 = self.ff1.fsf1
        fsf1.invokeFactory('FormStringField', 'fsf', title='a string field')

        res = self.ff1.fgFieldsDisplayList(objTypes=['FormStringField'])
        self.assertEqual(len(res), 3)

        res = self.ff1.mailer.fieldsDisplayList()
        self.assertEqual(len(res), 4)

    def testFgFieldsDisplayOnly(self):
        """ test EasyForm's fgFields displayOnly option """

        ff = self.ff1
        fields = get_fields(ff)
        res = dict([(i, fields[i]) for i in fields])
        len1 = len(res)

        # TODO add label fields
        # ff.invokeFactory('FormLabelField', 'lf')
        # ff.invokeFactory('FormRichLabelField', 'rlf')

        # when displayOnly==True, fgFields should not return label fields
        self.assertEqual(len(get_fields(ff)), len1)
        # when displayOnly is omitted, all fields should be returned
        self.assertEqual(len(ff.fgFields()), len1 + 2)

    def test_Validate(self):
        """ Test required field validation """

        request = self.fakeRequest(topic='test subject')

        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertTrue(errors['replyto'])
        self.assertTrue(errors['comments'])
        self.assertTrue(errors.get('topic') is None)

        request = self.fakeRequest(
            topic='test subject', replyto='testtest.org', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertTrue(errors['replyto'])

        request = self.fakeRequest(
            topic='test subject', replyto='test@test.org', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(errors, {})

        # since that should have validated, it should have been mailed
        self.assertTrue(self.messageText.find('Reply-To: test@test.org') > 0)

    def test_selfValidate(self):
        """ Test field self validation """

        request = self.fakeRequest(topic='test subject ')
        errors = self.ff1.topic.fgvalidate(request)
        self.assertEqual(errors, {})

    def test_i18nTitleValidate(self):
        """ Test field self validation with required non-ASCII field title """

        self.ff1.topic.setTitle(
            'Effacer les entr\xc3\xa9es sauvegard\xc3\xa9es')
        request = self.fakeRequest()
        errors = self.ff1.topic.fgvalidate(request)
        self.assertTrue('topic' in errors)

    def test_CustomValidation(self):
        """ test to make sure the custom TALES validation works
        """

        # test in field context (field validating itself)

        # make sure it fails when test is invalid
        request = self.fakeRequest(topic='test subject ')
        self.ff1.topic.setFgTValidator('python: 1/0')
        self.assertRaises(
            ZeroDivisionError, self.ff1.topic.fgvalidate, request)

        # now for a more realistic custom validator
        self.ff1.topic.setFgTValidator(
            'python: test(value.find("test") >= 0, 0, "test is missing")')

        request = self.fakeRequest(topic='test subject ')
        errors = self.ff1.topic.fgvalidate(request)
        self.assertEqual(errors, {})

        request = self.fakeRequest(topic='no subject ')
        errors = self.ff1.topic.fgvalidate(request)
        self.assertTrue(errors['topic'] == 'test is missing')

        # also check in form context (form validating field)

        request = self.fakeRequest(
            topic='test subject', replyto='test@test.org', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(errors, {})

        request = self.fakeRequest(
            topic='no subject', replyto='test@test.org', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertTrue(errors['topic'] == 'test is missing')

    def testHtmlValue(self):
        """ Test field htmlValue method """

        request = self.fakeRequest(topic='test subject')
        self.assertEqual(self.ff1['topic'].htmlValue(request), 'test subject')

        # test html escaping
        request = self.fakeRequest(topic='test < & > subject')
        self.assertEqual(self.ff1['topic'].htmlValue(
            request), 'test &lt; &amp; &gt; subject')

        # test list cleanup
        request = self.fakeRequest(topic=['one', ])
        self.assertEqual(self.ff1['topic'].htmlValue(request), "'one'")

        # test non-list non-cleanup
        request = self.fakeRequest(topic="['one',]")
        self.assertEqual(self.ff1['topic'].htmlValue(request), "['one',]")

        # test eol encoding
        request = self.fakeRequest(comments='one\ntwo')
        self.assertEqual(
            self.ff1['comments'].htmlValue(request), '<div>one<br />two</div>')

    def testHtmlValueSelectionField(self):
        """ Test field htmlValue method of selection field """

        self.ff1.invokeFactory('FormSelectionField', 'fsf')
        self.ff1.fsf.fgVocabulary = ('1|one', '2|two', '3|three',)

        # first test inside the vocabulary
        request = self.fakeRequest(fsf='2')
        val = self.ff1['fsf'].htmlValue(request)
        self.assertEqual(val, 'two')

        # now, outside the vocabulary
        request = self.fakeRequest(fsf='7')
        val = self.ff1['fsf'].htmlValue(request)
        self.assertEqual(val, '7')

        # now, outside the vocabulary;
        # make sure it's html escaped
        request = self.fakeRequest(fsf='&')
        val = self.ff1['fsf'].htmlValue(request)
        self.assertEqual(val, '&amp;')

    def testHtmlValueMultiSelectionField(self):
        """ Test field htmlValue method of multi-selection field """

        self.ff1.invokeFactory('FormMultiSelectionField', 'fsf')
        self.ff1.fsf.fgVocabulary = ('1|one', '2|two', '3|three',)

        # first test inside the vocabulary
        request = self.fakeRequest(fsf=['2', '3', ''])
        val = self.ff1['fsf'].htmlValue(request)
        self.assertEqual(val, 'two, three')

        # now, outside the vocabulary
        request = self.fakeRequest(fsf=['7', ''])
        val = self.ff1['fsf'].htmlValue(request)
        self.assertEqual(val, '7')

        # now, outside the vocabulary;
        # make sure it's html-escaped
        request = self.fakeRequest(fsf=['&', ''])
        val = self.ff1['fsf'].htmlValue(request)
        self.assertEqual(val, '&amp;')

        # now, mixed
        request = self.fakeRequest(fsf=['1', '7', ''])
        val = self.ff1['fsf'].htmlValue(request)
        self.assertEqual(val, 'one, 7')

    def testHtmlValueDateField(self):
        """ Test field htmlValue method of date field """

        self.ff1.invokeFactory('FormDateField', 'fdf')

        # good date
        request = self.fakeRequest(fdf='2007/01/01 00:00')
        self.ff1['fdf'].htmlValue(request)

        # bad date
        request = self.fakeRequest(fdf='2007/01/00 00:00')
        self.ff1['fdf'].htmlValue(request)

    def testMissingAdapter(self):
        """ test response to missing adapter -- should not fail """

        # Note: this test logs a warning message; it does not indicate test
        # failure

        self.ff1.invokeFactory('FormSaveDataAdapter', 'saver')

        self.ff1.setActionAdapter(('bogus',))
        self.assertEqual(self.ff1.actionAdapter, ('bogus',))

        request = self.fakeRequest(
            topic='test subject', replyto='test@test.org', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(errors, {})

    def testSimpleCall(self):
        """ test calling form """

        # this proves we can call a form from the test
        # harness.

        self.ff1()

    def testCallWithGoodOverride(self):
        """ test calling form with good code in override """

        # This test isn't as trivial as it looks. It makes sure the
        # call to the override doesn't clobber the expression context
        # and foul template evaluation.

        self.ff1.setOnDisplayOverride('python: 1')
        self.ff1()

    def testCallWithAfterValidationOverride(self):
        """ test calling form with good code in AfterValidationOverride """

        self.ff1.setAfterValidationOverride('python: 1')
        self.ff1()

    def testCallWithBadOverride(self):
        """ test calling form with bad code in override """

        self.ff1.setOnDisplayOverride('python: 1/0')
        self.assertRaises(ZeroDivisionError, self.ff1)

    def testCatalogCleanup(self):
        """ Test to make sure portalfactory isn't leaving
            ghost entries in portal_catalog """

        # look up all the FormStringFields
        sfb = self.portal.portal_catalog(portal_type='FormStringField')
        # There should be two
        self.assertEqual(len(sfb), 2)

    def testEmptySelectionField(self):
        """ test for issue  # 80: No radio button selected: AttributeError
            'NoneType' object has no attribute 'encode' """

        self.ff1.invokeFactory('FormSelectionField', 'fsf')
        self.ff1.fsf.fgVocabulary = ('one', 'two', 'three', 'four', 'five')
        self.ff1.fsf.setFgFormat('radio')

        request = self.fakeRequest(
            topic='test subject', replyto='test@test.org', comments='test comments')
        self.ff1.fsf.htmlValue(request)

    def testEmptyMultiSelectionField(self):
        """ check MSF for issue  # 80: No radio button selected: AttributeError
            'NoneType' object has no attribute 'encode' """

        self.ff1.invokeFactory('FormMultiSelectionField', 'fsf')
        self.ff1.fsf.fgVocabulary = ('one', 'two', 'three', 'four', 'five')
        self.ff1.fsf.setFgFormat('checkbox')

        request = self.fakeRequest(
            topic='test subject', replyto='test@test.org', comments='test comments')
        self.ff1.fsf.htmlValue(request)

    def testTrailSpacesValidation(self):
        """ check MSF for issue  # 79:
            Trailing space in recipient address of mailer causes validation failure"""

        request = self.fakeRequest(
            topic='test subject', replyto='test@test.org ', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(errors, {})

    def testTrailSpacesSave(self):
        """ We really don't want to save or act on trailing spaces in inputs """

        request = self.fakeRequest(
            topic='test subject', replyto='test@test.org ', comments='test comments')
        self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(request.form['replyto'], 'test@test.org')

    def testWhiteSpaceInRequired(self):
        """ white-space only shouldn't validate in required fields  """

        request = self.fakeRequest(
            topic='test subject', replyto='test@test.org', comments='\n')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(len(errors), 1)

        request = self.fakeRequest(
            topic='test subject', replyto='test@test.org', comments='xx\n')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(len(errors), 0)

        request = self.fakeRequest(
            topic='   ', replyto='test@test.org', comments='xx\n')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(len(errors), 1)

        request = self.fakeRequest(
            topic='x   ', replyto='test@test.org', comments='xx\n')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(len(errors), 0)

    def testListMarshallFix(self):
        """ test fgvalidate fix for odd Zope list marshalling """

        self.ff1.invokeFactory('FormMultiSelectionField', 'fsf')
        self.ff1.fsf.fgVocabulary = ('one', 'two', 'three', 'four', 'five')
        self.ff1.fsf.setFgFormat('checkbox')
        request = self.fakeRequest(
            topic='test subject', replyto='test@test.org', comments='test', fsf=['one', 'two', ''])
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(errors, {})
        # print request.form['fsf']
        self.assertEqual(len(request.form['fsf']), 2)

    def testFieldEnables(self):
        """ Test TAL field enables/disables  """

        # crude test: see if fields are enabled or not by testing
        # whether or not they get validated.

        # no enabling condition: should not validate
        request = self.fakeRequest(
            topic='test subject', replyto='test@test.org')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(len(errors), 1)

        # explicity enabling: should not validate
        self.ff1.comments.setFgTEnabled('python: True')
        request = self.fakeRequest(
            topic='test subject', replyto='test@test.org')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(len(errors), 1)

        # explicity disabling: should validate
        self.ff1.comments.setFgTEnabled('python: False')
        request = self.fakeRequest(
            topic='test subject', replyto='test@test.org')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(len(errors), 0)

        # now, let's count fields returned by fgFields

        # no enabling condition: all fields should be in list
        self.ff1.comments.setFgTEnabled('')
        request = self.fakeRequest()
        fields = self.ff1.fgFields(request=request)
        self.assertEqual(len(fields), 3)

        # explicitly enabled: all fields should be in list
        self.ff1.comments.setFgTEnabled('python: True')
        request = self.fakeRequest()
        fields = self.ff1.fgFields(request=request)
        self.assertEqual(len(fields), 3)

        # explicitly disabled: one less fields should be in list
        self.ff1.comments.setFgTEnabled('python: False')
        request = self.fakeRequest()
        fields = self.ff1.fgFields(request=request)
        self.assertEqual(len(fields), 2)

    def testTranslationBasics(self):
        """ Sanity check i18n setup against some known translations
            This test will fail if .mo files don't exist.
        """

        from collective.easyform import EasyFormMessageFactory as _
        from zope.i18n import translate

        # test with:
        # msgid "clear-save-input"

        msg = _(u'clear-save-input', u'Clear Saved Input')

        xlation = translate(msg, target_language='en')
        self.assertEqual(xlation, u'Clear Saved Input')

        # xlation = translate(msg, target_language='fr')
        # self.assertEqual( xlation, 'Effacer les entr\xc3\xa9es sauvegard\xc3\xa9es'.decode('utf8') )

        # xlation = translate(msg, target_language='de')
        # self.assertEqual( xlation, 'Die gespeicherten Eingaben l\xc3\xb6schen'.decode('utf8') )

    def testDateValidation(self):
        """ Dates should be validated """

        self.ff1.invokeFactory('FormDateField', 'fdf')

        # set non-date fields in request
        request = self.fakeRequest(
            topic='test subject', replyto='test@test.org ', comments='test comments')

        # try with no date at all. should validate, since fdf isn't required
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(request.form['replyto'], 'test@test.org')
        self.assertEqual(errors, {})

        # try with good date. should validate.
        request = self.fakeRequest(
            topic='test subject', replyto='test@test.org ', comments='test comments',
            fdf='2007/02/20')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(request.form['replyto'], 'test@test.org')
        self.assertEqual(errors, {})

        # try with bad date. should not validate.
        request = self.fakeRequest(
            topic='test subject', replyto='test@test.org ', comments='test comments',
            fdf='2007/02/31', fdf_year='2007', fdf_month='02', fdf_day='31')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(request.form['replyto'], 'test@test.org')
        self.assertEqual(len(errors), 1)

        # try required and bad date. should not validate.
        self.ff1.fdf.setRequired(True)
        request = self.fakeRequest(
            topic='test subject', replyto='test@test.org ', comments='test comments',
            fdf='2007/02/31', fdf_year='2007', fdf_month='02', fdf_day='31')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(request.form['replyto'], 'test@test.org')
        self.assertEqual(len(errors), 1)

        # try required and no date. should not validate.
        self.ff1.fdf.setRequired(True)
        request = self.fakeRequest(
            topic='test subject', replyto='test@test.org ', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(request.form['replyto'], 'test@test.org')
        self.assertEqual(len(errors), 1)

        # try required and good date. should validate.
        self.ff1.fdf.setRequired(True)
        request = self.fakeRequest(
            topic='test subject', replyto='test@test.org ', comments='test comments',
            fdf='2007/02/21', fdf_year='2007', fdf_month='02', fdf_day='21')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(request.form['replyto'], 'test@test.org')
        self.assertEqual(errors, {})

    def testEmptyHiddenLinesField(self):
        """ test empty, hidden lines field, issue 151 """

        self.ff1.invokeFactory('FormLinesField', 'flf')
        self.ff1.flf.setHidden(True)
        # smoke test
        self.ff1()

    def testActionAdapterReturns(self):
        """ test to make sure that the return status of action adapters is handled right """

        # Script code to imitate a Custom Script adapter
        # script returning something other than a dict
        non_error_script = """
        # # Python Script
        # #bind container=container
        # #bind context=context
        # #bind subpath=traverse_subpath
        # #parameters=fields, easyform, request
        # #title=Succesfully working script returning error
        # #

        return False
        """

        # Script code to imitate a Custom Script adapter
        # script returning a dict and using context.FORM_ERROR_MARKER
        error_script = """
        # # Python Script
        # #bind container=container
        # #bind context=context
        # #bind subpath=traverse_subpath
        # #parameters=fields, easyform, request
        # #title=Succesfully working script returning error
        # #

        return {context.FORM_ERROR_MARKER:'an error message'}
        """

        # we'll need privileges to create a script adapter
        self.loginAsPortalOwner()

        # create three adapters
        self.ff1.invokeFactory('FormSaveDataAdapter', 'saver1')
        saver1 = self.ff1.saver1
        self.ff1.invokeFactory('FormCustomScriptAdapter', 'cscript')
        cscript = self.ff1.cscript
        self.ff1.invokeFactory('FormSaveDataAdapter', 'saver2')
        saver2 = self.ff1.saver2
        self.ff1.setActionAdapter(('saver1', 'cscript', 'saver2'))
        self.assertEqual(
            self.ff1.actionAdapter, ('saver1', 'cscript', 'saver2'))

        # fake request to fake post
        request = self.fakeRequest(
            topic='test subject', replyto='test@test.org', comments='test comments')

        # Run a script that returns a non-error status;
        # Something should be saved to both savers,
        # and errors should be an empty dict.
        cscript.setScriptBody(stripWhiteSpace(non_error_script))
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(errors, {})
        self.assertEqual(saver1.itemsSaved(), 1)
        self.assertEqual(saver2.itemsSaved(), 1)

        # Run a script that returns an error status;
        # we should see an error status from the validator,
        # and action adapter execution should short-circuit
        # with only the first saver's onSuccess being executed.
        #
        # This will also demonstrate that the dictionary returned
        # by the action adapter is returned by fgvalidate
        # and that FORM_ERROR_MARKER is available as an
        # attribute of the context.
        cscript.setScriptBody(stripWhiteSpace(error_script))
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(
            errors, {cscript.FORM_ERROR_MARKER: 'an error message'})
        self.assertEqual(saver1.itemsSaved(), 2)
        self.assertEqual(saver2.itemsSaved(), 1)

    def testCSRF(self):
        """ test csrf protection """

        # for this test, we need a bit more serious request simulation
        from ZPublisher.HTTPRequest import HTTPRequest
        from ZPublisher.HTTPResponse import HTTPResponse
        environ = {}
        environ.setdefault('SERVER_NAME', 'foo')
        environ.setdefault('SERVER_PORT', '80')
        environ.setdefault('REQUEST_METHOD', 'POST')
        request = HTTPRequest(sys.stdin,
                              environ,
                              HTTPResponse(stdout=sys.stdout))

        request.form = {
            'topic': 'test subject',
            'replyto': 'test@test.org',
            'comments': 'test comments',
        }

        self.ff1.CSRFProtection = True

        self.assertRaises(zExceptions.Forbidden, self.ff1.fgvalidate, request)

        # with authenticator... no error
        tag = AuthenticatorView('context', 'request').authenticator()
        token = tag.split('"')[5]
        request.form['_authenticator'] = token
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(errors, {})

        # sneaky GET request
        environ['REQUEST_METHOD'] = 'GET'
        request = HTTPRequest(sys.stdin,
                              environ,
                              HTTPResponse(stdout=sys.stdout))
        self.assertRaises(zExceptions.Forbidden, self.ff1.fgvalidate, request)

        # bad authenticator
        request.form['_authenticator'] = 'inauthentic'
        request = HTTPRequest(sys.stdin,
                              environ,
                              HTTPResponse(stdout=sys.stdout))
        self.assertRaises(zExceptions.Forbidden, self.ff1.fgvalidate, request)

    def testBooleanRequired(self):
        """ test for issue  # 202: bad enforcement of required
            on boolean fields
        """

        self.ff1.invokeFactory('FormBooleanField', 'fbf')
        self.ff1.fbf.setRequired(True)

        request = self.fakeRequest(
            topic='test subject', replyto='test@test.org', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual(errors, {})

    def testJSTranslate(self):
        """test the browser view that supplies translations for javascript
        """
        jsvars = getMultiAdapter(
            (self.ff1, self.fakeRequest()),
            name='pfg_javascript_variables.js'
        )
        res = jsvars()
        self.assertEqual(res.find('pfgQEdit.messages = {'), 0)
        self.assertTrue(res.find("ORDER_MSG: 'Order'") > 0)


def test_suite():
    from unittest import TestSuite  # , makeSuite
    suite = TestSuite()
    # suite.addTest(makeSuite(TestFunctions))
    return suite



# -*- coding: utf-8 -*-

from AccessControl import getSecurityManager
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ZPublisher.mapply import mapply
from collective.easyform import easyformMessageFactory as _
from collective.easyform.api import DollarVarReplacer
from collective.easyform.api import filter_fields
from collective.easyform.api import get_actions
from collective.easyform.api import get_expression
from collective.easyform.api import get_fields
from collective.easyform.interfaces import IActionExtender
from collective.easyform.interfaces import IEasyFormForm
from collective.easyform.interfaces import IFieldExtender
from logging import getLogger
from plone.app.z3cform.inline_validation import InlineValidationView
from plone.autoform.form import AutoExtensibleForm
from plone.z3cform import interfaces
from plone.z3cform.layout import FormWrapper
from z3c.form import button
from z3c.form import form
from z3c.form.interfaces import DISPLAY_MODE
from z3c.form.interfaces import HIDDEN_MODE
from z3c.form.interfaces import IErrorViewSnippet
from zope.component import getMultiAdapter
from zope.i18nmessageid import MessageFactory
from zope.interface import implements
from zope.schema import ValidationError
from zope.schema import getFieldsInOrder
from zope.event import notify
from z3c.form.events import DataExtractedEvent

logger = getLogger('collective.easyform')
PMF = MessageFactory('plone')


class EasyFormForm(AutoExtensibleForm, form.Form):
    implements(IEasyFormForm)

    """
    EasyForm form
    """
    template = ViewPageTemplateFile('easyform_form.pt')
    ignoreContext = True
    css_class = 'easyformForm'
    thanksPage = False

    @property
    def method(self):
        return self.context.method

    @property
    def default_fieldset_label(self):
        return self.context.default_fieldset_label or super(EasyFormForm, self).default_fieldset_label

    def action(self):
        """ Redefine <form action=''> attribute.
        """
        action = getattr(self.context, 'formActionOverride')
        if action:
            action = get_expression(self.context, action)
        if not action:
            action = self.context.absolute_url()
        if self.context.forceSSL:
            action = action.replace('http://', 'https://')
        return action

    @property
    def enable_form_tabbing(self):
        return self.context.form_tabbing

    @property
    def enable_unload_protection(self):
        return self.context.unload_protection

    @property
    def enableCSRFProtection(self):
        return self.context.CSRFProtection

    @property
    def label(self):
        return self.thanksPage and self.context.thankstitle or self.context.Title()

    @property
    def description(self):
        return self.thanksPage and self.context.thanksdescription or self.context.Description()

    @property
    def prologue(self):
        return self.thanksPage and self.thanksPrologue or self.context.formPrologue.output

    @property
    def epilogue(self):
        return self.thanksPage and self.thanksEpilogue or self.context.formEpilogue.output

    @property
    def schema(self):
        return get_fields(self.context)

    def updateServerSideData(self, data):
        for fname in self.schema:
            field = self.schema[fname]
            efield = IFieldExtender(field)
            serverSide = getattr(efield, 'serverSide', False)
            if not serverSide:
                continue
            fdefault = field.default
            TDefault = getattr(efield, 'TDefault', None)
            value = get_expression(
                self.context, TDefault) if TDefault else fdefault
            data[fname] = value
        return data

    def processActions(self, data):
        # get a list of adapters with no duplicates, retaining order
        actions = getFieldsInOrder(get_actions(self.context))
        for name, action in actions:
            if not action.required:
                continue
            # Now, see if we should execute it.
            # Check to see if execCondition exists and has contents
            execCondition = IActionExtender(action).execCondition
            if execCondition:
                doit = get_expression(self.context, execCondition)
            else:
                doit = True
            if doit and hasattr(action, 'onSuccess'):
                result = action.onSuccess(data, self.request)
                if isinstance(result, dict) and len(result):
                    return result

    def setDisplayMode(self, mode):
        self.mode = mode
        for widget in self.widgets.values():
            widget.mode = mode
        self.updateWidgets()
        for group in self.groups:
            group.widgets.mode = mode
            for field in group.widgets:
                del group.widgets[field]
            group.widgets.update()

    def setErrorsMessage(self, errors):
        for field in errors:
            if field not in self.widgets:
                continue
            error = ValidationError()
            error.doc = lambda: errors[field]
            view = getMultiAdapter(
                (error, self.request, self.widgets[
                    field], self.widgets[field].field, self, self.context),
                IErrorViewSnippet)
            view.update()
            self.widgets.errors += (view,)
            self.widgets[field].error = view
        self.status = self.formErrorsMessage

    @button.buttonAndHandler(PMF(u'Submit'), name='submit', condition=lambda form: not form.thanksPage)
    def handleSubmit(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        data = self.updateServerSideData(data)
        errors = self.processActions(data)
        if errors:
            return self.setErrorsMessage(errors)
        thanksPageOverride = self.context.thanksPageOverride
        if thanksPageOverride:
            thanksPageOverrideAction = self.context.thanksPageOverrideAction
            thanksPage = get_expression(self.context, thanksPageOverride)
            if thanksPageOverrideAction == 'redirect_to':
                self.request.response.redirect(thanksPage)
            elif thanksPageOverrideAction == 'traverse_to':
                thanksPage = self.context.restrictedTraverse(
                    thanksPage.encode('utf-8'))
                thanksPage = mapply(
                    thanksPage, self.request.args, self.request).encode('utf-8')
                self.request.response.write(thanksPage)
        else:
            self.thanksPage = True
            replacer = DollarVarReplacer(data).sub
            self.thanksPrologue = self.context.thanksPrologue and replacer(
                self.context.thanksPrologue.output)
            self.thanksEpilogue = self.context.thanksEpilogue and replacer(
                self.context.thanksEpilogue.output)
            if not self.context.showAll:
                self.fields = self.setThanksFields(self.base_fields, data)
                for name in list(self.widgets.keys()):
                    if name not in self.fields:
                        del self.widgets[name]
                for group in self.groups:
                    group.fields = self.setThanksFields(
                        self.base_groups.get(group.label), data)
                    for name in list(group.widgets.keys()):
                        if name not in group.fields:
                            del group.widgets[name]
            self.setDisplayMode(DISPLAY_MODE)
            self.updateActions()

    @button.buttonAndHandler(_(u'Reset'), name='reset', condition=lambda form: form.context.useCancelButton or form.thanksPage)
    def handleReset(self, action):
        self.request.response.redirect(self.nextURL())

    def nextURL(self):
        return self.context.absolute_url()

    def setOmitFields(self, fields):
        omit = []
        for fname in fields:
            field = fields[fname].field
            efield = IFieldExtender(field)
            TEnabled = getattr(efield, 'TEnabled', None)
            serverSide = getattr(efield, 'serverSide', False)
            if TEnabled and not get_expression(self.context, TEnabled) or serverSide:
                omit.append(fname)
        if omit:
            fields = fields.omit(*omit)
        return fields

    def setThanksFields(self, fields, data):
        omit = filter_fields(self.context, self.schema, data, omit=True)
        new_fields = []
        for fname, field in fields.items():
            if field.mode == HIDDEN_MODE:
                field.mode = DISPLAY_MODE
            new_fields.append(field)
        fields = fields.__class__(*new_fields)
        if omit:
            fields = fields.omit(*omit)

        # We need extra handling to get the correct order.
        showFields = getattr(self.context, "showFields", [])
        if showFields:
            # showFields may contain fields from other fieldsets,
            # which we should first remove from this list.
            field_order = [
                fname for fname in showFields if fname in fields.keys()
            ]
            fields = fields.select(*field_order)
        return fields

    def updateFields(self):
        # if self.thanksPage:
        #     return
        super(EasyFormForm, self).updateFields()
        if not hasattr(self, 'base_fields'):
            self.base_fields = self.fields
        if not hasattr(self, 'base_groups'):
            self.base_groups = dict([(i.label, i.fields) for i in self.groups])
        self.fields = self.setOmitFields(self.base_fields)
        for group in self.groups:
            group.fields = self.setOmitFields(
                self.base_groups.get(group.label))

    def updateActions(self):
        super(EasyFormForm, self).updateActions()
        if 'submit' in self.actions:
            if self.context.submitLabelOverride:
                self.actions['submit'].title = get_expression(
                    self.context, self.context.submitLabelOverride)
            else:
                self.actions['submit'].title = self.context.submitLabel
        if 'reset' in self.actions:
            self.actions['reset'].title = self.context.resetLabel

    def formMaybeForceSSL(self):
        """ Redirect to an https:// URL if the 'force SSL' option is on.

            However, don't do so for those with rights to edit the form,
            to avoid making the form uneditable if SSL isn't configured
            properly.  These users will still get an SSL-ified form
            action for when the form is submitted.
        """
        if self.context.forceSSL and not getSecurityManager().checkPermission('cmf.ModifyPortalContent', self):
            # Make sure we're being accessed via a secure connection
            if self.request['SERVER_URL'].startswith('http://'):
                secure_url = self.request['URL'].replace('http://', 'https://')
                self.request.response.redirect(
                    secure_url, status='movedtemporarily')

    def update(self):
        '''See interfaces.IForm'''
        self.formMaybeForceSSL()
        super(EasyFormForm, self).update()

    def header_injection(self):
        tal_expression = self.context.headerInjection
        if not tal_expression:
            return ''
        header_to_inject = get_expression(self.context, tal_expression)
        if  isinstance(header_to_inject, unicode):
            header_to_inject = header_to_inject.encode('utf-8')
        return header_to_inject

class EasyFormFormWrapper(FormWrapper):
    form = EasyFormForm
    index = ViewPageTemplateFile('easyform_layout.pt')

    def header_injection(self):
        header_injection = self.form_instance.header_injection()
        return header_injection

    def css_class(self):
        css_class = None
        if self.form_instance.thanksPage:
            css_class = u"easyform-thankspage"
        return css_class

EasyFormView = EasyFormFormWrapper


class EasyFormFormEmbedded(EasyFormForm):

    """
    EasyForm form embedded
    """
    template = ViewPageTemplateFile('easyform_form_embedded.pt')

class EasyFormInlineValidationView(InlineValidationView):

    def __call__(self, fname=None, fset=None):
        self.context = EasyFormForm(self.context, self.request)
        return super(EasyFormInlineValidationView, self).__call__(fname, fset)


from setuptools import setup, find_packages

version = '1.0.0'

setup(name='collective.easyform',
      version=version,
      description="Forms for Plone",
      long_description=open("README.rst").read() +
      "\n" + open("CHANGES.rst").read(),
      # Get more strings from
      # http://pypi.python.org/pypi?:action=list_classifiers
      classifiers=[
          "Development Status :: 5 - Production/Stable",
          "Framework :: Plone",
          "Framework :: Plone :: 4.3",
          "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
          "Programming Language :: Python",
          "Programming Language :: Python :: 2.7",
      ],
      keywords='form plone ttw',
      author='Roman Kozlovskyi',
      author_email='krzroman@gmail.com',
      maintainer='Plone Release Team',
      maintainer_email='releaseteam@plone.org',
      url='https://github.com/collective/collective.easyform',
      license='GPLv2',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'plone.app.dexterity',
          'plone.app.textfield',
          'plone.autoform',
          'plone.dexterity',
          'plone.namedfile',
          'plone.schemaeditor==2.0.7',
          'plone.supermodel',
          # -*- Extra requirements: -*-
      ],
      extras_require={
          'recaptcha': [
              'plone.formwidget.recaptcha'
          ],
          'test': [
              'plone.app.contenttypes',
              'plone.app.testing[robot]',
              'plone.app.robotframework',
          ]
      },
      entry_points="""
      # -*- Entry points: -*-

      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



# -*- coding: utf-8 -*-
#
# Test EasyForm initialisation and set-up
#

try:
    from Products.CMFDefault.exceptions import EmailAddressInvalid
except ImportError:
    from Products.CMFPlone.RegistrationTool import EmailAddressInvalid
from Products.validation import validation
from ZPublisher.BaseRequest import BaseRequest
from collective.easyform import validators
from collective.easyform.api import get_fields
from collective.easyform.api import set_fields
from collective.easyform.browser.view import EasyFormForm
from collective.easyform.interfaces import IFieldExtender
from collective.easyform.tests import base
from os.path import dirname
from os.path import join
from plone.namedfile.file import NamedFile
from z3c.form.interfaces import IFormLayer
from zope.component import getUtility
from zope.component.interfaces import ComponentLookupError
from zope.interface import classImplements

IFieldValidator = validators.IFieldValidator

FORM_DATA = {
    'topic': u'test subject',
    'replyto': u'test@test.org',
    'comments': u'test comments',
}


class TestBaseValidators(base.EasyFormTestCase):

    """ test base validators """

    def afterSetUp(self):
        base.EasyFormTestCase.afterSetUp(self)
        self.folder.invokeFactory('EasyForm', 'ff1')
        self.ff1 = getattr(self.folder, 'ff1')
        self.ff1.CSRFProtection = False  # no csrf protection
        classImplements(BaseRequest, IFormLayer)
        from collective.easyform.validators import update_validators
        update_validators()

        request = self.app.REQUEST
        for i in FORM_DATA:
            request.form['form.widgets.{0}'.format(i)] = FORM_DATA[i]

    def test_defaultvalidator(self):
        view = self.ff1.restrictedTraverse('view')
        form = view.form_instance
        form.update()

        data, errors = form.extractData()
        self.assertEqual(errors, ())
        self.assertEqual(data, FORM_DATA)

    def test_basevalidator(self):
        fields = get_fields(self.ff1)
        IFieldExtender(fields['replyto']).validators = ["isEmail"]
        set_fields(self.ff1, fields)
        view = self.ff1.restrictedTraverse('view')
        form = view.form_instance
        form.update()

        data, errors = form.extractData()
        self.assertEqual(errors, ())
        self.assertEqual(data, FORM_DATA)

    def test_basevalidator2(self):
        fields = get_fields(self.ff1)
        IFieldExtender(fields['comments']).validators = ["isInt", "isURL"]
        set_fields(self.ff1, fields)
        view = self.ff1.restrictedTraverse('view')
        form = view.form_instance
        form.update()

        data, errors = form.extractData()
        self.assertEqual(len(errors), 1)

    def test_talvalidator(self):
        fields = get_fields(self.ff1)
        IFieldExtender(
            fields['comments']).TValidator = "python: value == 'comments'"
        set_fields(self.ff1, fields)
        view = self.ff1.restrictedTraverse('view')
        form = view.form_instance
        form.update()

        data, errors = form.extractData()
        self.assertEqual(errors, ())
        self.assertEqual(data, FORM_DATA)

    def test_talvalidator2(self):
        fields = get_fields(self.ff1)
        IFieldExtender(fields['comments']).TValidator = "python: !value"
        set_fields(self.ff1, fields)
        view = self.ff1.restrictedTraverse('view')
        form = view.form_instance
        form.update()

        data, errors = form.extractData()
        self.assertEqual(len(errors), 1)


class TestSingleFieldValidator(base.EasyFormTestCase):

    """ test validator in form outside of fieldset

    The test methods are reused in TestFieldsetValidator.
    They use the same field, except that one has it in a fieldset.
    """
    schema_fixture = "single_field.xml"

    def afterSetUp(self):
        self.folder.invokeFactory("EasyForm", "ff1")
        self.ff1 = getattr(self.folder, "ff1")
        self.ff1.CSRFProtection = False  # no csrf protection
        self.ff1.showAll = True
        self.portal.invokeFactory("File", "easyform_default_fields.xml")

        field_template = self.portal["easyform_default_fields.xml"]
        with open(join(dirname(__file__), "fixtures", self.schema_fixture)) as f:
            filecontent = NamedFile(f.read(), contentType="application/xml")
        field_template.file = filecontent
        classImplements(BaseRequest, IFormLayer)
        validators.update_validators()

    def LoadRequestForm(self, **kwargs):
        request = self.layer["request"]
        request.form.clear()
        prefix = "form.widgets."
        for key in kwargs.keys():
            request.form[prefix + key] = kwargs[key]
        return request

    def test_get_default(self):
        # With a GET, we should see the default value in the form.
        request = self.LoadRequestForm()
        request.method = "GET"
        form = EasyFormForm(self.ff1, request)()
        self.assertNotIn('Required input is missing.', form)
        self.assertIn('value="foo@example.org"', form)

    def test_required(self):
        data = {"replyto": ""}
        request = self.LoadRequestForm(**data)
        request.method = "POST"
        form = EasyFormForm(self.ff1, request)()
        self.assertIn('Required input is missing.', form)
        self.assertNotIn('Invalid email address.', form)

    def test_validator_in_fieldset(self):
        data = {
            "replyto": "bad email address",
        }
        request = self.LoadRequestForm(**data)
        request.method = "POST"
        form = EasyFormForm(self.ff1, request)()
        self.assertNotIn('Required input is missing.', form)
        self.assertIn('Invalid email address.', form)


class TestFieldsetValidator(TestSingleFieldValidator):

    """ test validator in fieldset

    This reuses the test methods from TestSingleFieldValidator.
    """
    schema_fixture = "fieldset_with_single_field.xml"


class TestCustomValidators(base.EasyFormTestCase):

    """ test our validators """

    def ttest_inExNumericRange(self):
        v = validation.validatorFor('inExNumericRange')
        self.assertEqual(v(10, minval=1, maxval=20), 1)
        self.assertEqual(v('10', minval=1, maxval=20), 1)
        self.assertEqual(v('1', minval=1, maxval=20), 1)
        self.assertEqual(v('20', minval=1, maxval=20), 1)
        self.assertNotEqual(v(0, minval=1, maxval=5), 1)
        self.assertNotEqual(v(6, minval=1, maxval=5), 1)
        self.assertNotEqual(v(4, minval=5, maxval=3), 1)

    def ttest_isNotTooLong(self):
        v = validation.validatorFor('isNotTooLong')
        self.assertEqual(v('', maxlength=20), 1)
        self.assertEqual(v('1234567890', maxlength=20), 1)
        self.assertEqual(v('1234567890', maxlength=10), 1)
        self.assertEqual(v('1234567890', maxlength=0), 1)
        self.assertNotEqual(v('1234567890', maxlength=9), 1)
        self.assertNotEqual(v('1234567890', maxlength=1), 1)

    def test_isChecked(self):
        v = getUtility(IFieldValidator, name='isChecked')
        self.assertEqual(v('1'), None)
        self.assertNotEqual(v('0'), None)

    def test_isUnchecked(self):
        v = getUtility(IFieldValidator, name='isUnchecked')
        self.assertEqual(v('0'), None)
        self.assertNotEqual(v('1'), None)

    def test_isNotLinkSpam(self):
        v = getUtility(IFieldValidator, name='isNotLinkSpam')
        good = """I am link free and proud of it"""
        bad1 = """<a href="mylink">Bad.</a>"""
        bad2 = """http://bad.com"""
        bad3 = """www.Bad.com"""
        bad = (bad1, bad2, bad3)

        self.assertEqual(v(good), None)
        for b in bad:
            self.assertNotEqual(
                v(b), None, '"{0}" should be considered a link.'.format(b))

    def ttest_isNotTooLong2(self):
        v = validation.validatorFor('isNotTooLong')
        v.maxlength = 10
        self.assertEqual(v('abc'), 1)
        self.assertNotEqual(v('abcdefghijklmnopqrstuvwxyz'), 1)

        # there was a bug where widget.maxlength could possibly be defined as
        # '' which means calling int(widget.maxlength) would fail

        class Mock(object):
            pass
        field = Mock()
        field.widget = Mock()
        field.widget.maxlength = ''

        self.assertEqual(v('abc', field=field), 1)

    def test_isEmail(self):
        v = getUtility(IFieldValidator, name='isValidEmail')
        self.assertEqual(v('hi@there.com'), None)
        self.assertEqual(v('one@u.washington.edu'), None)
        self.assertRaises(EmailAddressInvalid, v, '@there.com')

    def test_isCommaSeparatedEmails(self):
        v = getUtility(IFieldValidator, name='isCommaSeparatedEmails')
        self.assertEqual(v('hi@there.com,another@two.com'), None)
        self.assertEqual(
            v('one@u.washington.edu,  two@u.washington.edu'), None)
        self.assertNotEqual(v('abc@plone.org; xyz@plone.org'), None)


class TestCustomValidatorMessages(base.EasyFormTestCase):

    """ Test friendlier validation framework """

    def test_stringValidators(self):
        """ Test string validation
        """
        from collective.easyform.validators import update_validators
        update_validators()

        validator = lambda n: getUtility(IFieldValidator, name=n)
        validate = lambda n, v: validator(n) and validator(n)(v)

        self.assertRaises(
            ComponentLookupError, validate, 'noValidator', 'test')

        self.assertNotEqual(validate('isEmail', 'test'), None)

        self.assertEqual(validate('isEmail', 'test@test.com'), None)

        self.assertEqual(validate('isZipCode', '12345'), None)
        self.assertNotEqual(validate('isZipCode', '12345-1234'), None)
        # self.assertEqual(validate('isZipCode', '12345-1234'), None)
        # Canadian zip codes
        # self.assertEqual(validate('isZipCode', 'T2X 1V4'), None)
        # self.assertEqual(validate('isZipCode', 'T2X1V4'), None)
        # self.assertEqual(validate('isZipCode', 't2x 1v4'), None)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestBaseValidators))
    suite.addTest(makeSuite(TestCustomValidators))
    suite.addTest(makeSuite(TestCustomValidatorMessages))
    return suite



# -*- coding: utf-8 -*-
#
# Integration tests for interaction with GenericSetup infrastructure
#

from collective.easyform import content
from collective.easyform import interfaces
from collective.easyform.tests import base
from zope.component import getMultiAdapter
from zope.interface.verify import verifyClass
from zope.interface.verify import verifyObject


class TestFormGenInterfaces(base.EasyFormTestCase):

    """ Some boilerplate-ish tests to confirm that that classes
        and instances confirm to the interface contracts intended.
    """

    def afterSetUp(self):
        base.EasyFormTestCase.afterSetUp(self)

        # add form folder for use in tests
        self.folder.invokeFactory('EasyForm', 'ff1')

    def testBrowserViewClassInterfaces(self):
        """Some basic boiler plate testing of interfaces and classes"""
        # verify IEasyFormExportView
        # self.assertTrue(
        #     interfaces.IEasyFormExportView.implementedBy(exportimport.EasyFormExportView))
        # self.assertTrue(
        #     verifyClass(interfaces.IEasyFormExportView,
        #     exportimport.EasyFormExportView))

    def testBrowserViewObjectsVerify(self):
        # verify views are objects of the expected class, verified
        # implementation
        form_folder_export = getMultiAdapter(
            (self.folder.ff1, self.app.REQUEST), name='export-form-folder')
        # self.assertTrue(isinstance(
        #     form_folder_export, exportimport.EasyFormExportView))
        self.assertTrue(verifyObject(interfaces.IEasyFormExportView,
                        form_folder_export))

    def testContentClassInterfaces(self):
        self.assertTrue(
            interfaces.IEasyFormFieldset.implementedBy(content.fieldset.FieldsetFolder))
        self.assertTrue(
            verifyClass(interfaces.IEasyFormFieldset, content.fieldset.FieldsetFolder))


def test_suite():
    from unittest import TestSuite  # , makeSuite
    suite = TestSuite()
    # suite.addTest(makeSuite(TestFormGenInterfaces))
    return suite



# -*- coding: utf-8 -*-

try:
    from Products.validation import validation
    from Products.validation.validators.BaseValidators import baseValidators
except ImportError:
    validation = {}
    baseValidators = []

try:
    from Products.CMFDefault.exceptions import EmailAddressInvalid
except ImportError:
    from Products.CMFPlone.RegistrationTool import EmailAddressInvalid

from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.utils import getToolByName
from collective.easyform import easyformMessageFactory as _
from collective.easyform.interfaces import IFieldValidator
from types import BooleanType
from types import StringTypes
from zope.component import getUtility
from zope.component import provideUtility


def isValidEmail(value):
    """Check for the user email address"""
    portal = getUtility(ISiteRoot)
    reg_tool = getToolByName(portal, 'portal_registration')
    if not (value and reg_tool.isValidEmail(value)):
        raise EmailAddressInvalid


def isCommaSeparatedEmails(value):
    """Check for one or more E-Mail Addresses separated by commas"""
    portal = getUtility(ISiteRoot)
    reg_tool = getToolByName(portal, 'portal_registration')
    for v in value.split(','):
        if not reg_tool.isValidEmail(v.strip()):
            return _(u'Must be a valid list of email addresses (separated by commas).')


def isChecked(value):
    if (type(value) == BooleanType) and value or (type(value) in StringTypes) and (value == '1'):
        return
    return _(u'Must be checked.')


def isUnchecked(value):
    if (type(value) == BooleanType) and not value or (type(value) in StringTypes) and (value == '0'):
        return
    return _(u'Must be unchecked.')


def isNotLinkSpam(value):
    # validation is optional and configured on the field
    bad_signs = ('<a ', 'www.', 'http:', '.com', )
    value = value.lower()
    for s in bad_signs:
        if s in value:
            return _('Links are not allowed.')

# Base validators


def update_validators():
    if validation and baseValidators:
        def method(name):
            def validate(value):
                if isinstance(value, unicode):
                    value = value.encode('utf-8')
                res = validation(name, value)
                if res != 1:
                    return res
            return validate
        for validator in baseValidators:
            provideUtility(method(validator.name),
                           provides=IFieldValidator, name=validator.name)
update_validators()
